import requests


class KoodousError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class KoodousNotFound(KoodousError):
    pass


class Koodous(object):
    def __init__(self, token=None):
        self.token = token
        self.base_url = "https://developer.koodous.com"

    def _query(self, url, params={}):
        headers = {
            "Authorization": "Token " + self.token,
            'User-Agent': 'Harpoon (https://github.com/Te-k/harpoon)'
        }
        r = requests.get(self.base_url + url, params=params, headers=headers)
        if r.status_code == 404:
            raise KoodousNotFound()
        elif r.status_code != 200:
            raise KoodousError("Invalid HTTP code {} - {}".format(r.status_code, r.text))
        return r.json()

    def sha256(self, hash):
        return self._query("/apks/" + hash)

    def search(self, query):
        return self._query("/apks/", {'search': query})

    def download(self, hash):
        headers = {
            "Authorization": "Token " + self.token,
            'User-Agent': 'Harpoon (https://github.com/Te-k/harpoon)'
        }
        r = requests.get(
            self.base_url + "/apks/" + hash + "/download/",
            headers=headers)
        if r.status_code == 404:
            raise KoodousNotFound()
        elif r.status_code != 200:
            raise KoodousError("Invalid HTTP code {}".format(r.status_code))
        return r.content

    def analysis(self, hash):
        return self._query("/apks/" + hash + "/analysis")

    def account(self):
        return self._query("/account/")



import requests


class PulseDiveError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class PulseDiveNotFound(PulseDiveError):
    pass


class PulseDive(object):
    def __init__(self, api_key=None):
        self.api_key = api_key
        self.base_url = "https://pulsedive.com/api/"

    def _query(self, url, params):
        if self.api_key:
            params["key"] = self.api_key

        headers = {'User-Agent': 'Harpoon (https://github.com/Te-k/harpoon)'}
        r = requests.get(
            self.base_url + url,
            params=params,
            headers=headers
        )
        if r.status_code == 404:
            raise PulseDiveNotFound("Not found")
        elif r.status_code != 200:
            raise PulseDiveError("Bad HTTP Response {}".format(r.status_code))
        return r.json()

    def indicators_by_value(self, val, historical=False):
        """
        This query is identical to querying by indicator ID, but the
        information is retrieved using the indicator value. Querying
        links and properties by indicator value won't work.
        """
        params = {'indicator': val}
        if historical:
            params['historical'] = "1"
        return self._query("info.php", params)

    def indicators_by_value_links(self, val):
        return self._query("info.php", {'indicator': val, "get": "links"})

    def indicators_by_value_properties(self, val):
        return self._query("info.php", {'indicator': val, "get": "properties"})

    def threat(self, threat):
        return self._query("info.php", {'threat': threat})

    def threat_indicators(self, tid):
        return self._query("info.php", {'tid': tid, 'get': 'links'})



import base64

import requests


class PhishtankError(Exception):
    pass


class Phishtank:
    def __init__(self, key=None):
        self.url = "https://checkurl.phishtank.com/checkurl/"
        self.headers = {
            "User-Agent": "Harpoon (https://github.com/Te-k/harpoon)",
            "Content-Type": "application/x-www-form-urlencoded"
        }
        self.key = key

    def query(self, url):
        post_data = {
            "url": base64.b64encode(url.encode("utf-8")),
            "format": "json",
        }
        if self.key:
            post_data["app_key"] = self.key
        r = requests.post(
            self.url,
            data=post_data,
            headers=self.headers)
        if r.status_code != 200:
            raise PhishtankError("Wrong HTTP code {}".format(r.status_code))
        return r.json()



import json
from datetime import datetime

import requests


class RobtexError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class Robtex(object):
    """
    Library to request Robtex API
    https://www.robtex.com/api/
    TODO : support the API
    """
    def _request(self, path):
        r = requests.get("https://freeapi.robtex.com" + path)
        if r.status_code != 200:
            raise RobtexError("Wrong HTTP status code %i" % r.status_code)
        else:
            data = r.json()
            if "status" not in data:
                raise RobtexError("Wrong message format")
            else:
                if data["status"] != "ok":
                    raise RobtexError("Wrong status code %s" % data["status"])
                else:
                    return data

    def _parse_pdns(self, data):
        return [json.loads(a) for a in data.split()]

    def get_ip_info(self, ip):
        """
        Request info on an IPv4
        """
        data = self._request("/ipquery/%s" % ip)
        for d in ["pas", "pash", "act", "acth"]:
            if d in data:
                for dd in data[d]:
                    if "t" in dd:
                        dd["date"] = datetime.fromtimestamp(dd["t"])
        return data

    def get_asn_info(self, asn):
        """
        Get ASN info
        """
        return self._request("/asquery/%i" % asn)

    def get_pdns_domain(self, domain):
        """
        Get passive DNS info on a domain
        """
        r = requests.get("https://freeapi.robtex.com/pdns/forward/%s" % domain)
        if r.status_code != 200:
            raise RobtexError("Wrong HTTP status code %i" % r.status_code)
        else:
            data = self._parse_pdns(r.text)
            for d in data:
                if "time_first" in d:
                    d["time_first_o"] = datetime.fromtimestamp(d["time_first"])
                if "time_last" in d:
                    d["time_last_o"] = datetime.fromtimestamp(d["time_last"])

            return data

    def get_pdns_ip(self, ip):
        """
        Get passive DNS info on an IP address
        """
        r = requests.get("https://freeapi.robtex.com/pdns/reverse/%s" % ip)
        if r.status_code != 200:
            raise RobtexError("Wrong HTTP status code %i" % r.status_code)
        else:
            data = self._parse_pdns(r.text)
            for d in data:
                if "time_first" in d:
                    d["time_first_o"] = datetime.fromtimestamp(d["time_first"])
                if "time_last" in d:
                    d["time_last_o"] = datetime.fromtimestamp(d["time_last"])

            return data



#! /usr/bin/env python
import json
import sys

import requests

from harpoon.commands.base import Command


class CommandQuad9(Command):
    """
    # Quad9

    Check if a domain is blocked by Quad9 https://quad9.net/

    `harpoon quad9 DOMAIN`
    """

    name = "quad9"
    description = "Check if a domain is blocked by Quad9"
    config = {"Quad9": []}

    def add_arguments(self, parser):
        parser.add_argument("DOMAIN", help="Domain to be checked")
        parser.add_argument("--type", "-t", default="A", help="DNS Type")
        parser.add_argument(
            "--verbose", "-v", action="store_true", help="Display results"
        )
        self.parser = parser

    def run(self, args, plugins):
        params = {
            "name": args.DOMAIN,
            "type": args.type,
            "ct": "application/dns-json",
        }
        r = requests.get("https://dns.quad9.net:5053/dns-query", params=params)
        if r.status_code != 200:
            print("Problem querying quad9 :(")
            sys.exit(1)
        res = r.json()
        if res["Status"] == 3:
            if "Authority" in res:
                print("{} - NXDOMAIN".format(args.DOMAIN))
            else:
                print("{} - BLOCKED".format(args.DOMAIN))
        else:
            print("{} - NOT BLOCKED".format(args.DOMAIN))
        if args.verbose:
            print(json.dumps(r.json(), indent=4))

    def intel_domain(self, query, data):
        print("[+] Checking Quad9...")
        params = {
            "name": query,
            "type": "A",
            "ct": "application/dns-json",
        }
        r = requests.get("https://dns.quad9.net:5053/dns-query", params=params)
        if r.status_code != 200:
            print("Quad9 query failed")
        else:
            res = r.json()
            if res["Status"] == 3:
                if "Authority" not in res:
                    data["reports"].append(
                        {
                            "title": "Domain blocked by Quad9",
                            "date": "",
                            "source": "Quad9",
                            "url": "",
                        }
                    )



#! /usr/bin/env python
import json

from harpoon.commands.base import Command
from harpoon.lib.opencage import OpenCage, OpenCageError


class CommandOpenCageInfo(Command):
    """
    # OpenCage plugin

    **Query OpenCage geocoder API**

    https://opencagedata.com/

    * Forward Geocoding : `harpoon opencage search QUERY`
    * Reverse Geocoding : `harpoon opencage reverse LATITUDE LONGITUDE`
    """

    name = "opencage"
    description = "Forward/Reverse Geocoding using OpenCage"
    config = {"OpenCage": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "search", help="Search coordinates of an address or location"
        )
        parser_a.add_argument("QUERY", help="Query (Address, Location, ...)")
        parser_a.set_defaults(subcommand="search")
        parser_b = subparsers.add_parser(
            "reverse", help="Request the nearest address for coordinates"
        )
        parser_b.add_argument("LATITUDE", help="Latitude")
        parser_b.add_argument("LONGITUDE", help="Longitude")
        parser_b.set_defaults(subcommand="reverse")
        self.parser = parser

    def run(self, args, plugins):
        geocoder = OpenCage(key=self._config_data["OpenCage"]["key"])
        if "subcommand" in args:
            if args.subcommand == "search":
                try:
                    infos = geocoder.geocode(args.QUERY)
                except OpenCageError:
                    print("Invalid request")
                else:
                    print(
                        json.dumps(
                            infos, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            elif args.subcommand == "reverse":
                try:
                    infos = geocoder.reverse(args.LATITUDE, args.LONGITUDE)
                except OpenCageError:
                    print("Invalid request")
                else:
                    print(
                        json.dumps(
                            infos, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import json

from harpoon.commands.base import Command
from harpoon.lib.pithus import Pithus


class CommandPithus(Command):
    """
    # Pithus

    Queries the Pithus API

    * Query the database: `harpoon pithus report SHA256`
    * Query the status of an upload: `harpoon pithus status SHA256`
    * Upload a sample: `harpoon pithus upload FILEPATH`
    * Search the database: `harpoon pithus search SHA256`
    * Advanced search example: `harpoon pithus search 'domains: *google.com && permissions: *INTERNET'`

    See: beta.pithus.org for documentation on the search options
    """

    name = "pithus"
    description = "Search Pithus database for submitted APKs"
    config = {"Pithus": ["key", "url"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")

        # get request for a report
        parser_a = subparsers.add_parser("report", help="Retrieve a report from Pithus")
        parser_a.add_argument("SHA256", help="Hash of the sample")
        parser_a.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_a.set_defaults(subcommand="report")

        # get request for the status of the analysis of a report
        parser_b = subparsers.add_parser(
            "status", help="Retrieve the status of tasks for a given report"
        )
        parser_b.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_b.add_argument("SHA256", help="Hash of the sample")
        parser_b.set_defaults(subcommand="status")

        # post request to upload a sample
        parser_c = subparsers.add_parser("upload", help="Upload a sample on Pithus")
        parser_c.add_argument(
            "FILEPATH", help="Upload a sample to Pithus, provide a filepath"
        )
        parser_c.set_defaults(subcommand="upload")

        # post request for advanced search
        parser_d = subparsers.add_parser("search", help="Search in Pithus")
        parser_d.add_argument("QUERY", help="Search query, default is SHA256")
        parser_d.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_d.set_defaults(subcommand="search")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" not in args:
            self.parser.print_help()
        else:
            pithus = Pithus(self._config_data["Pithus"])

            if args.subcommand == "report":
                res = pithus.report(args.SHA256)
                if args.json:
                    print(json.dumps(res, indent=4))
                else:
                    pithus.pretty_print(res, "report")
            elif args.subcommand == "status":
                res = pithus.status(args.SHA256)
                if args.json:
                    print(json.dumps(res, indent=4))
                else:
                    pithus.pretty_print(res, "status")
            elif args.subcommand == "upload":
                with open(args.FILEPATH, "rb") as f:
                    data = f.read()
                pithus.upload(data)
            elif args.subcommand == "search":
                res = pithus.search(args.QUERY)
                if args.json:
                    print(json.dumps(res, indent=4))
                else:
                    pithus.pretty_print(res, "search")
            else:
                self.parser.print_help()

    # TODO : intel



#! /usr/bin/env python
import json

import pytz
from dateutil.parser import parse

from harpoon.commands.base import Command
from harpoon.lib.pulsedive import PulseDive, PulseDiveError
from harpoon.lib.utils import unbracket


class CommandPulseDive(Command):
    """
    # PulseDive plugin

    Queries https://pulsedive.com/ API

    * Search on a domain or IP : `harpoon pulsedive indicator DOMAIN`
    * Search for domains related to an IP : `harpoon pulsedive links IP`
    * Search on a threat : `harpoon pulsedive threat zeus`
    """

    name = "pulsedive"
    description = "Request PulseDive API"
    config = {"PulseDive": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "indicator", help="Request info on an indicator"
        )
        parser_a.add_argument("IOC", help="Indicator")
        parser_a.add_argument(
            "--history", "-H", action="store_true", help="Also queries historical data"
        )
        parser_a.set_defaults(subcommand="indicator")
        parser_b = subparsers.add_parser(
            "links", help="Identify links with other indicators"
        )
        parser_b.add_argument("IOC", help="Indicator")
        parser_b.set_defaults(subcommand="links")
        parser_c = subparsers.add_parser(
            "properties", help="Identify properties of an indicator"
        )
        parser_c.add_argument("IOC", help="Indicator")
        parser_c.set_defaults(subcommand="properties")
        parser_d = subparsers.add_parser("threat", help="Get information on a threat")
        parser_d.add_argument("THREAT", help="Threat such as zeus")
        parser_d.set_defaults(subcommand="threat")
        self.parser = parser

    def run(self, args, plugins):
        p = PulseDive(self._config_data["PulseDive"]["key"])
        if "subcommand" in args:
            if args.subcommand == "indicator":
                res = p.indicators_by_value(
                    unbracket(args.IOC), historical=args.history
                )
                print(json.dumps(res, sort_keys=True, indent=4))
            elif args.subcommand == "links":
                res = p.indicators_by_value_links(args.IOC)
                print(json.dumps(res, sort_keys=True, indent=4))
            elif args.subcommand == "properties":
                res = p.indicators_by_value_properties(args.IOC)
                print(json.dumps(res, sort_keys=True, indent=4))
            elif args.subcommand == "threat":
                res = p.threat(args.THREAT)
                print(json.dumps(res, sort_keys=True, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_domain(self, query, data):
        pd = PulseDive(self._config_data["PulseDive"]["key"])
        print("[+] Checking PulseDive...")
        try:
            res = pd.indicators_by_value(unbracket(query), historical=True)
            if "properties" in res:
                if "dns" in res["properties"]:
                    for ip in res["properties"]["dns"]:
                        if ip["name"] == "A":
                            data["passive_dns"].append(
                                {
                                    "ip": ip["value"],
                                    "first": parse(ip["stamp_seen"]).astimezone(
                                        pytz.utc
                                    ),
                                    "last": None,
                                    "source": "PulseDive",
                                }
                            )
                if "threats" in res:
                    if len(res["threats"]) > 0:
                        data["reports"].append(
                            {
                                "date": None,
                                "source": "PulseDive",
                                "title": "Identified as related to {}".format(
                                    ", ".join([r["name"] for r in res["threats"]])
                                ),
                                "url": "",
                            }
                        )
            res = pd.indicators_by_value_links(unbracket(query))
            if "Related URLs" in res:
                for url in res["Related URLs"]:
                    data["urls"].append(
                        {
                            "source": "PulseDive",
                            "url": url["indicator"],
                            "ip": "",
                            "date": parse(url["stamp_linked"]).astimezone(pytz.utc),
                        }
                    )
        except PulseDiveError:
            pass

    def intel_ip(self, query, data):
        pd = PulseDive(self._config_data["PulseDive"]["key"])
        print("[+] Checking PulseDive...")
        try:
            res = pd.indicators_by_value(unbracket(query), historical=True)
            if "attributes" in res:
                if "port" in res["attributes"]:
                    for p in res["attributes"]["port"]:
                        data["ports"].append(
                            {"port": p, "source": "PulseDive", "info": ""}
                        )
            if "properties" in res:
                if "dns" in res["properties"]:
                    for ip in res["properties"]["dns"]:
                        if ip["name"] == "A":
                            data["passive_dns"].append(
                                {
                                    "domain": ip["value"],
                                    "first": parse(ip["stamp_seen"]).astimezone(
                                        pytz.utc
                                    ),
                                    "last": None,
                                    "source": "PulseDive",
                                }
                            )
                if "threats" in res:
                    if len(res["threats"]) > 0:
                        data["reports"].append(
                            {
                                "date": None,
                                "source": "PulseDive",
                                "title": "Identified as related to {}".format(
                                    ", ".join([r["name"] for r in res["threats"]])
                                ),
                                "url": "",
                            }
                        )
            res = pd.indicators_by_value_links(unbracket(query))
            if "Related URLs" in res:
                for url in res["Related URLs"]:
                    data["urls"].append(
                        {
                            "source": "PulseDive",
                            "url": url["indicator"],
                            "ip": "",
                            "date": parse(url["stamp_linked"]).astimezone(pytz.utc),
                        }
                    )
        except PulseDiveError:
            pass



#! /usr/bin/env python
import json
import time

import pytz
from dateutil.parser import parse

from harpoon.commands.base import Command
from harpoon.lib.urlscan import UrlScan, UrlScanError, UrlScanQuotaExceeded


class CommandUrlscan(Command):
    """
    # url scan

    Allows to search and scan urls using https://urlscan.io/

    * Query the database : `harpoon urlscan search DOMAIN`
    * View an analysis : `harpoon urlscan view UID`
    """

    name = "urlscan"
    description = "Search and submit urls to urlscan.io"
    # Key is optional
    config = {"UrlScan": []}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("search", help="Search in urlscan")
        parser_a.add_argument("QUERY", help="DOMAIN to be queried")
        parser_a.add_argument(
            "--raw", "-r", action="store_true", help="Shows raw results"
        )
        parser_a.set_defaults(subcommand="search")
        parser_b = subparsers.add_parser(
            "list", help="Search list of domains or IPs in urlscan"
        )
        parser_b.add_argument("FILE", help="File containing IPs or domains")
        parser_b.set_defaults(subcommand="list")
        parser_c = subparsers.add_parser("view", help="View urlscan analysis")
        parser_c.add_argument("UID", help="UId of the analysis")
        parser_c.set_defaults(subcommand="view")
        parser_d = subparsers.add_parser("quota", help="Show quota")
        parser_d.set_defaults(subcommand="quota")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            # Optional key
            try:
                key = self._config_data["UrlScan"]["key"]
                if key.strip() != "":
                    us = UrlScan(key)
                else:
                    us = UrlScan()
            except KeyError:
                us = UrlScan()
            if args.subcommand == "search":
                # Search
                try:
                    res = us.search(args.QUERY)
                except UrlScanError as e:
                    print("Error: {}".format(e.message))
                else:
                    if args.raw:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        if len(res["results"]) > 0:
                            for r in res["results"]:
                                print(
                                    "{} - {} - {} - https://urlscan.io/result/{}".format(
                                        r["task"]["time"],
                                        r["page"]["url"],
                                        r["page"]["ip"] if "ip" in r["page"] else "",
                                        r["_id"],
                                    )
                                )
                        else:
                            print("No results for this query")
            elif args.subcommand == "view":
                try:
                    print(json.dumps(us.view(args.UID), sort_keys=True, indent=4))
                except UrlScanError as e:
                    print("Error: {}".format(e.message))
            elif args.subcommand == "list":
                with open(args.FILE) as f:
                    data = f.read().split("\n")
                for d in data:
                    d = d.strip()
                    if d == "":
                        continue
                    print("##### {}".format(d))
                    try:
                        res = us.search(d)
                    except UrlScanQuotaExceeded as e:
                        duration = int(e.message[-13:-9]) + 10
                        print("Out of quota, waiting for {} seconds".format(duration))
                        time.sleep(duration)
                        res = us.search(d)
                    if "results" in res:
                        if len(res["results"]) > 0:
                            for r in res["results"]:
                                print(
                                    "{} - {} - {} - https://urlscan.io/result/{}".format(
                                        r["task"]["time"],
                                        r["page"]["url"],
                                        r["page"]["ip"] if "ip" in r["page"] else "",
                                        r["_id"],
                                    )
                                )
                        else:
                            print("Nothing found")
                    else:
                        print("Nothing found")
                    # brief sleeping time to avoid overloading URL Scan
                    time.sleep(0.5)
            elif args.subcommand == "quota":
                if us.api_key:
                    print(json.dumps(us.quota(), sort_keys=True, indent=4))
                else:
                    print("You need to configure a UrlScan key to check your quota")
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type in ["domain", "ip"]:
            print("[+] Checking UrlScan...")
            us = UrlScan()
            res = us.search(query)
            if "results" in res:
                for r in res["results"]:
                    data["urls"].append(
                        {
                            "date": parse(r["task"]["time"]).astimezone(pytz.utc),
                            "url": r["page"]["url"],
                            "ip": r["page"]["ip"] if "ip" in r["page"] else "",
                            "source": "UrlScan",
                        }
                    )



#! /usr/bin/env python3
import json

import pytz
from dateutil.parser import parse

from harpoon.commands.base import Command
from harpoon.lib.threatcrowd import ThreatCrowd, ThreatCrowdError
from harpoon.lib.utils import unbracket


class CommandThreatCrowd(Command):
    """
    # ThreatCrowd

    See: https://github.com/AlienVault-OTX/ApiV2

    With the ThreatCrowd API you can search for:

    * Domains
    * IP Addreses
    * E-mail adddresses
    * Filehashes
    * Antivirus detections


    * Query for an email: `harpoon threatcrowd --email EMAIL`
    * Query for a domain: `harpoon threatcrowd --domain DOMAIN`
    * Query for a IP: `harpoon threatcrowd --ip IP`
    * Query for a antivirus: `harpoon threatcrowd --antivirus MALWARE` (ex: plugx)
    * Query for a filehash: `harpoon threatcrowd --file HASH`
    """

    name = "threatcrowd"
    description = "Request the ThreatCrowd API"
    config = {"ThreatCrowd": []}

    def add_arguments(self, parser):
        parser.add_argument("--email", "-e", help="Query an email")
        parser.add_argument("--ip", "-i", help="Query an IP address")
        parser.add_argument("--domain", "-d", help="Query a domain")
        parser.add_argument(
            "--antivirus", "-a", help="Query an antivirus for malware hashes"
        )
        parser.add_argument("--file", "-f", help="Query for a file hash")
        self.parser = parser

    def pretty_print(self, data):
        print(json.dumps(data, indent=4, sort_keys=True))

    def run(self, args, plugins):
        tc = ThreatCrowd()
        try:
            if args.ip:
                self.pretty_print(tc.ip(unbracket(args.ip)))
            elif args.email:
                self.pretty_print(tc.email(args.email))
            elif args.domain:
                self.pretty_print(tc.domain(unbracket(args.domain)))
            elif args.antivirus:
                self.pretty_print(tc.antivirus(args.antivirus))
            elif args.file:
                self.pretty_print(tc.file(args.file))
            else:
                self.parser.print_help()
        except ThreatCrowdError as e:
            print("Query failed: {}".format(e.message))

    def intel(self, type, query, data):
        if type == "domain":
            print("[+] Checking ThreatCrowd...")
            tc = ThreatCrowd()
            try:
                res = tc.domain(query)
                if "resolutions" in res:
                    for d in res["resolutions"]:
                        if d["ip_address"] not in ["-", ""]:
                            data["passive_dns"].append(
                                {
                                    "ip": d["ip_address"],
                                    "first": parse(d["last_resolved"]).astimezone(
                                        pytz.utc
                                    ),
                                    "last": parse(d["last_resolved"]).astimezone(
                                        pytz.utc
                                    ),
                                    "source": "ThreatCrowd",
                                }
                            )
                if "hashes" in res:
                    for h in res["hashes"]:
                        data["malware"].append(
                            {"hash": h, "source": "ThreatCrowd", "date": ""}
                        )
            except ThreatCrowdError as e:
                print("Connection to ThreatCrowd failed: {}".format(e.message))
        elif type == "ip":
            print("[+] Checking ThreatCrowd...")
            tc = ThreatCrowd()
            try:
                res = tc.ip(query)
                if "resolutions" in res:
                    for d in res["resolutions"]:
                        data["passive_dns"].append(
                            {
                                "domain": d["domain"].strip(),
                                "first": parse(d["last_resolved"]).astimezone(pytz.utc),
                                "last": parse(d["last_resolved"]).astimezone(pytz.utc),
                                "source": "ThreatCrowd",
                            }
                        )
                if "hashes" in res:
                    for h in res["hashes"]:
                        data["malware"].append(
                            {"hash": h, "source": "ThreatCrowd", "date": ""}
                        )
            except ThreatCrowdError as e:
                print("Connection to ThreatCrowd failed: {}".format(e.message))
        elif type == "hash":
            print("[+] Checking ThreatCrowd...")
            if len(query.strip()) != 32:
                print("ThreatCrowd only accepts md5 ¯\\_(ツ)_/¯")
                return
            tc = ThreatCrowd()
            try:
                res = tc.file(query)
                if "domains" in res:
                    for d in res["domains"]:
                        data["network"].append(
                            {
                                "host": d,
                                "source": "ThreatCrowd",
                                "url": res["permalink"],
                            }
                        )
                if "ips" in res:
                    for d in res["ips"]:
                        data["network"].append(
                            {
                                "host": d,
                                "source": "ThreatCrowd",
                                "url": res["permalink"],
                            }
                        )
            except ThreatCrowdError as e:
                print("Connection to ThreatCrowd failed: {}".format(e.message))



#! /usr/bin/env python
import consolemd

from harpoon.commands.base import Command


class CommandHelp(Command):
    """
    # Help Command

    **Show help for a command**
    Example : `harpoon help config`
    """

    name = "help"
    description = "Give help on an Harpoon command"

    def add_arguments(self, parser):
        parser.add_argument(
            "COMMAND", help="Show the help of the given command", nargs="?"
        )

    def run(self, args, plugins, parser):
        if args.COMMAND in plugins:
            renderer = consolemd.Renderer()
            # Remove empty space at the beginning of lines
            content = [
                item.strip() for item in plugins[args.COMMAND].__doc__.splitlines()
            ]
            renderer.render("\n".join(content))
        else:
            parser.print_help()



#! /usr/bin/env python
import glob
import os
import shutil
import socket
import subprocess
import sys
import urllib
import urllib.request

import geoip2.database
import pyasn
from IPy import IP

from harpoon.commands.asn import CommandAsn
from harpoon.commands.base import Command
from harpoon.lib.utils import is_ip, unbracket


class CommandIp(Command):
    """
        # IP

        **Gathers information on an IP address**

        Get information on an IP:
        ```
    harpoon ip 172.34.127.2
    MaxMind: Located in None, United States
    MaxMind: ASN21928, T-Mobile USA, Inc.
    ASN 21928 - T-MOBILE-AS21928 - T-Mobile USA, Inc., US (range 172.32.0.0/11)

    Censys:     https://censys.io/ipv4/172.34.127.2
    Shodan:     https://www.shodan.io/host/172.34.127.2
    IP Info:    http://ipinfo.io/172.34.127.2
    BGP HE:     https://bgp.he.net/ip/172.34.127.2
    IP Location:    https://www.iplocation.net/?query=172.34.127.2
        ```
    """

    name = "ip"
    description = "Gather information on an IP address"
    config = None
    update_needed = True
    geocity = "/usr/share/GeoIP/GeoLite2-City.mmdb"
    geoasn = "/usr/share/GeoIP/GeoLite2-ASN.mmdb"
    asnname = os.path.join(os.path.expanduser("~"), ".config/harpoon/asnnames.csv")
    asncidr = os.path.join(os.path.expanduser("~"), ".config/harpoon/asncidr.dat")
    specific_ips = os.path.join(os.path.expanduser("~"), ".config/harpoon/iplist.csv")

    def add_arguments(self, parser):
        parser.add_argument("IP", help="IP address")
        self.parser = parser

    def update(self):
        print("Downloading AS data")
        file_name, headers = urllib.request.urlretrieve(
            "https://ftp.ripe.net/ripe/asnames/asn.txt"
        )
        if os.path.isfile(self.asnname):
            os.remove(self.asnname)
        fin = open(file_name, "r", encoding="latin-1", errors="ignore")
        fout = open(self.asnname, "w+")
        line = fin.readline()
        while line != "":
            asn = line.split(" ")[0]
            asname = " ".join(line.split(" ")[1:]).strip()
            fout.write("{}|{}\n".format(asn, asname))
            line = fin.readline()
        fin.close()
        fout.close()

        print("Downloading CIDR data")
        if os.path.isfile(self.asncidr):
            os.remove(self.asncidr)
        os.chdir("/tmp")
        subprocess.call(["pyasn_util_download.py", "--latest"])
        ls = glob.glob("rib*.bz2")[0]
        subprocess.call(["pyasn_util_convert.py", "--single", ls, "latest.dat"])
        shutil.move("latest.dat", self.asncidr)
        print("-asncidr.dat")

    def ip_get_asn(self, ip):
        """
        Take an IP address and returns the asn number and name
        returns {'asn': 1234, 'name': 'FORTUM-AS Fortum, FI'}
        If not found, returns {'asn': 0, 'name': ''}
        """
        self.check_geoipdb()
        try:
            asndb = geoip2.database.Reader(self.geoasn)
            res = asndb.asn(ip)
            asn = res.autonomous_system_number
            asn_name = res.autonomous_system_organization
        except geoip2.errors.AddressNotFoundError:
            # TODO: check in the other ASN db
            return {"asn": 0, "name": ""}
        return {"asn": asn, "name": asn_name}

    def check_geoipdb(self):
        """
        Check if the GeoIP database is present on the system
        Depending on geoipupdate version it can be stored in:
        /usr/share/GeoIP/
        /var/lib/GeoIP
        /usr/local/var/GeoIP/
        """
        if os.path.isfile("/usr/share/GeoIP/GeoLite2-City.mmdb"):
            self.geocity = "/usr/share/GeoIP/GeoLite2-City.mmdb"
        elif os.path.isfile("/var/lib/GeoIP/GeoLite2-City.mmdb"):
            self.geocity = "/var/lib/GeoIP/GeoLite2-City.mmdb"
        elif os.path.isfile("/usr/local/var/GeoIP/GeoLite2-City.mmdb"):
            self.geocity = "/usr/local/var/GeoIP/GeoLite2-City.mmdb"
        else:
            print("Impossible to find GeoIP db")
            print("Make sure you have geoipupdate correctly configured")
            sys.exit(1)

        if os.path.isfile("/usr/share/GeoIP/GeoLite2-ASN.mmdb"):
            self.geoasn = "/usr/share/GeoIP/GeoLite2-ASN.mmdb"
        elif os.path.isfile("/var/lib/GeoIP/GeoLite2-ASN.mmdb"):
            self.geoasn = "/var/lib/GeoIP/GeoLite2-ASN.mmdb"
        elif os.path.isfile("/usr/local/var/GeoIP/GeoLite2-ASN.mmdb"):
            self.geoasn = "/usr/local/var/GeoIP/GeoLite2-ASN.mmdb"
        else:
            print("Impossible to find GeoIP ASN db")
            print("Make sure you have geoipupdate correctly configured")
            print("It should include this configuration")
            print("EditionIDs GeoLite2-Country GeoLite2-City GeoLite2-ASN")
            sys.exit(1)

    def ipinfo(self, ip, dns=True):
        """
        Return information on an IP address
        {"asn", "asn_name", "city", "country"}
        """
        self.check_geoipdb()
        ipinfo = {}
        if dns:
            ipinfo["hostname"] = ""
            try:
                ipinfo["hostname"] = socket.gethostbyaddr(ip)[0]
            except socket.herror:
                pass
        try:
            citydb = geoip2.database.Reader(self.geocity)
            res = citydb.city(ip)
            ipinfo["city"] = res.city.name
            ipinfo["country"] = res.country.name
        except geoip2.errors.AddressNotFoundError:
            ipinfo["city"] = "Unknown"
            ipinfo["country"] = "Unknown"
        except FileNotFoundError:
            print(
                "GeoIP database not found, make sure you have correctly installed geoipupdate"
            )
            sys.exit(1)

        asninfo = self.ip_get_asn(ip)
        ipinfo["asn"] = asninfo["asn"]
        ipinfo["asn_name"] = asninfo["name"]
        ipinfo["specific"] = ""
        try:
            with open(self.specific_ips) as f:
                data = f.read().split("\n")
            for d in data:
                if d.strip().startswith(ip):
                    ipinfo["specific"] = d.split(",")[1].strip()
        except FileNotFoundError:
            pass
            # TODO: add private
        asnc = CommandAsn(self._config_data)
        res = asnc.asn_caida(ipinfo["asn"])
        ipinfo["asn_type"] = res["type"]
        return ipinfo

    def run(self, args, plugins):
        ip = unbracket(args.IP)
        if not is_ip(ip):
            print("Invalid IP address")
            sys.exit(1)
        # FIXME: move code here in a library
        ipinfo = self.ipinfo(ip)
        print("MaxMind: Located in %s, %s" % (ipinfo["city"], ipinfo["country"]))
        if ipinfo["asn"] == 0:
            print("MaxMind: IP not found in the ASN database")
        else:
            print("MaxMind: ASN%i, %s" % (ipinfo["asn"], ipinfo["asn_name"]))
            print("CAIDA Type: %s" % ipinfo["asn_type"])
        try:
            asndb2 = pyasn.pyasn(self.asncidr)
            res = asndb2.lookup(ip)
        except OSError:
            print("Configuration files are not available")
            print("Please run harpoon update before using harpoon")
            sys.exit(1)
        if res[1] is None:
            print("IP not found in ASN database")
        else:
            # Search for name
            f = open(self.asnname, "r")
            found = False
            line = f.readline()
            name = ""
            while not found and line != "":
                s = line.split("|")
                if s[0] == str(res[0]):
                    name = s[1].strip()
                    found = True
                line = f.readline()

            print("ASN %i - %s (range %s)" % (res[0], name, res[1]))
        if ipinfo["hostname"] != "":
            print("Hostname: %s" % ipinfo["hostname"])
        if ipinfo["specific"] != "":
            print("Specific: %s" % ipinfo["specific"])
        ipy = IP(ip)
        if ipy.iptype() == "PRIVATE":
            "Private IP"
        print("")
        if ipy.version() == 4:
            print("Censys:\t\thttps://censys.io/ipv4/%s" % ip)
            print("Shodan:\t\thttps://www.shodan.io/host/%s" % ip)
            print("IP Info:\thttp://ipinfo.io/%s" % ip)
            print("BGP HE:\t\thttps://bgp.he.net/ip/%s" % ip)
            print("IP Location:\thttps://www.iplocation.net/?query=%s" % ip)



#! /usr/bin/env python
import json

from harpoon.commands.base import Command
from harpoon.lib.malshare import (MalShare, MalShareFailed, MalShareNotFound,
                                  MalShareSampleMissing)
from harpoon.lib.utils import json_serial


class CommandMalShare(Command):
    """
    # MalShare

    **Requests information from MalShare (https://malshare.com/)**

    * `harpoon malshare search beget.tech` : Search in the database
    * `harpoon malshare hash 924c1fb188fb8dcbcee616308389fc22` : Information about a hash
    * `harpoon  malshare download dde72ae232dc63298465861482d7bb93 -o infected` : ;Download the sample
    """

    name = "malshare"
    description = "Requests MalShare database"
    config = {"MalShare": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Request info on a hash")
        parser_a.add_argument("HASH", help="Hash")
        parser_a.set_defaults(subcommand="hash")
        parser_b = subparsers.add_parser("download", help="Download the given hash")
        parser_b.add_argument("HASH", help="Hash")
        parser_b.add_argument("--output", "-o", help="Output file name")
        parser_b.set_defaults(subcommand="download")
        parser_e = subparsers.add_parser("search", help="Search in MalShare database")
        parser_e.add_argument("QUERY", help="query")
        parser_e.set_defaults(subcommand="search")
        self.parser = parser

    def run(self, args, plugins):
        ms = MalShare(self._config_data["MalShare"]["key"])
        if "subcommand" in args:
            if args.subcommand == "search":
                try:
                    res = ms.search(args.QUERY)
                except MalShareFailed:
                    print("File not found")
                else:
                    print(
                        json.dumps(
                            res,
                            sort_keys=True,
                            indent=4,
                            separators=(",", ": "),
                            default=json_serial,
                        )
                    )
            elif args.subcommand == "hash":
                try:
                    res = ms.file_info(args.HASH)
                except MalShareNotFound:
                    print("File not found")
                else:
                    print(
                        json.dumps(
                            res,
                            sort_keys=True,
                            indent=4,
                            separators=(",", ": "),
                            default=json_serial,
                        )
                    )
            elif args.subcommand == "download":
                try:
                    data = ms.download(args.HASH)
                except MalShareNotFound:
                    print("File not found")
                except MalShareSampleMissing:
                    print("Missing sample, it is a bug")
                else:
                    outfile = args.output if args.output else args.HASH
                    with open(outfile, "w") as f:
                        f.write(data)
                    print("Sample created %s" % outfile)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "hash":
            ms = MalShare(self._config_data["MalShare"]["key"])
            try:
                ms.file_info(query)
            except MalShareNotFound:
                pass
            else:
                data["samples"].append(
                    {
                        "date": None,
                        "source": "MalShare",
                        "url": "https://malshare.com/sample.php?action=detail&hash={}".format(
                            query
                        ),
                    }
                )



import urllib

import requests
from dateutil.parser import parse


class ArchiveOrg(object):
    """
    Object to download cache from archive.org
    """
    @staticmethod
    def snapshots(url):
        """
        Return a list of snapshots for a given url
        """
        # FIXME: report more snapshot through the Memento API
        r = requests.get(
            'http://archive.org/wayback/available?url=%s' %
            urllib.parse.quote(url))
        data = r.json()
        res = []
        if 'archived_snapshots' in data:
            for i in data['archived_snapshots']:
                res.append({
                    'url': data['url'],
                    'archive': data['archived_snapshots'][i]['url'],
                    'date': parse(data['archived_snapshots'][i]['timestamp'])
                })
        return res

    @staticmethod
    def download_cache(cache_url):
        """
        Download cache from a cache url
        """
        if cache_url.startswith('https://web.archive.org/web/') or \
                cache_url.startswith('http://web.archive.org/web/'):
            r = requests.get(cache_url)
            data = r.text
            t1 = data.find('<!-- End Wayback Rewrite JS Include -->')
            cached_data = '<!doctype html>\n<html>\n<head>' + data[t1+39:]
            if cache_url.startswith('http://'):
                date = parse(cache_url[27:41])
            else:
                date = parse(cache_url[28:42])
            return {
                'success': True,
                'data': cached_data,
                'date': date,
                'cacheurl': cache_url
            }
        else:
            return {'success': False}

    @staticmethod
    def cache(url):
        """
        Download an url from a cache
        """
        snapshots = ArchiveOrg.snapshots(url)
        if len(snapshots):
            last = sorted(snapshots, key=lambda x: x['date'], reverse=True)[0]
            return ArchiveOrg.download_cache(last['archive'])
        else:
            return {'success': False}

    @staticmethod
    def capture(url):
        """
        Capture url in Web Archive
        """
        # Copied from https://github.com/motherboardgithub/mass_archive/blob/master/mass_archive.py
        res = requests.get("https://web.archive.org/save/%s" % url)
        if res.status_code == 200:
            result = res.headers['Content-Location']
            internet_archive_url = "https://web.archive.org%s" % result
            return internet_archive_url
        else:
            raise Exception("Capture failed")



#! /usr/bin/env python
import json
import os

import pytz
from dateutil.parser import parse

from harpoon.commands.base import Command
from harpoon.lib.hybrid import HybridAnalysis, HybridAnalysisFailed
from harpoon.lib.utils import json_serial


class CommandHybridAnalysis(Command):
    """
    # HybridAnalysis

    **Query Hybrid Analysis Sandbox information (https://www.hybrid-analysis.com/)**

    Requests:
    * `harpoon hybrid hash 52c408a3dd38743a179ef800b390bbac` : Give information on a hash
    * `harpoon hybrid quota` : show quota information
    * `harpoon hybrid search host:95.181.53.78` : search in Hybrid Analysis samples
    * `harpoon analysis 52c408a3dd38743a179ef800b390bbac 100` : gived detailed sandbox report

    Search operators :
    * host:95.181.53.78
    * port:3448
    * domain:checkip.dyndns.org
    * vxfamily:upatre
    * indicatorid:network-6 (Show all reports matching 'Contacts Random Domain Names')
    * filetype:jar
    * filetype_tag:hwp
    * url:google
    * similar-to:59f629287c1ce2bcb39e4bc41633756e516bf04909750eba1cd2c787d02d3347
    * authentihash:hash
    * tag:teslacrypt
    """

    name = "hybrid"
    description = "Requests Hybrid Analysis platform"
    config = {"HybridAnalysis": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Request info on a hash")
        parser_a.add_argument("HASH", help="Hash")
        parser_a.set_defaults(subcommand="hash")
        parser_c = subparsers.add_parser(
            "domain", help="Search samples communicating with a domain"
        )
        parser_c.add_argument("DOMAIN", help="Domain")
        parser_c.set_defaults(subcommand="domain")
        parser_d = subparsers.add_parser(
            "ip", help="Search samples communicating with a, IP address"
        )
        parser_d.add_argument("IP", help="IP address")
        parser_d.set_defaults(subcommand="ip")
        parser_e = subparsers.add_parser("search", help="Search in HybridAnalysis")
        parser_e.add_argument("QUERY", help="query", nargs="+")
        parser_e.set_defaults(subcommand="search")
        parser_f = subparsers.add_parser("dl", help="Download a sample")
        parser_f.add_argument("HASH", help="SHA256 of the file")
        parser_f.set_defaults(subcommand="dl")
        self.parser = parser

    def run(self, args, plugins):
        ha = HybridAnalysis(self._config_data["HybridAnalysis"]["key"])
        if "subcommand" in args:
            if args.subcommand == "search":
                try:
                    data = {}
                    for d in args.QUERY:
                        if ":" in d:
                            dd = d.split(":")
                            data[dd[0]] = dd[1]
                    res = ha.search_terms(data)
                except HybridAnalysisFailed as e:
                    print("Query failed")
                    print(e)
                else:
                    print(
                        json.dumps(
                            res, indent=4, separators=(",", ": "), default=json_serial
                        )
                    )
            elif args.subcommand == "hash":
                try:
                    res = ha.search_hash(args.HASH)
                except HybridAnalysisFailed as e:
                    print("Query failed")
                    print(e)
                else:
                    print(
                        json.dumps(
                            res, indent=4, separators=(",", ": "), default=json_serial
                        )
                    )
            elif args.subcommand == "domain":
                try:
                    res = ha.search_terms({"domain": args.DOMAIN})
                except HybridAnalysisFailed as e:
                    print("Query failed")
                    print(e)
                else:
                    print(
                        json.dumps(
                            res, indent=4, separators=(",", ": "), default=json_serial
                        )
                    )
            elif args.subcommand == "ip":
                try:
                    res = ha.search_terms({"host": args.IP})
                except HybridAnalysisFailed as e:
                    print("Query failed")
                    print(e)
                else:
                    print(
                        json.dumps(
                            res, indent=4, separators=(",", ": "), default=json_serial
                        )
                    )
            elif args.subcommand == "dl":
                if os.path.isfile(args.HASH):
                    os.remove(args.HASH)
                try:
                    res = ha.overview_sample(args.HASH)
                except HybridAnalysisFailed as e:
                    print("Query failed")
                    print(e)
                else:
                    with open(args.HASH, "wb+") as f:
                        f.write(res)
                    print("Sample saved as {}".format(args.HASH))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        ha = HybridAnalysis(self._config_data["HybridAnalysis"]["key"])
        if type == "domain":
            print("[+] Checking HybridAnalysis...")
            try:
                res = ha.search_terms({"domain": query})
            except HybridAnalysisFailed:
                print("Query failed")
            else:
                for r in res["result"]:
                    data["malware"].append(
                        {
                            "source": "HybridAnalysis",
                            "hash": r["sha256"],
                            "date": parse(r["analysis_start_time"]).astimezone(
                                pytz.utc
                            ),
                        }
                    )
        elif type == "ip":
            print("[+] Checking HybridAnalysis...")
            try:
                res = ha.search_terms({"host": query})
            except HybridAnalysisFailed:
                print("Query failed")
            else:
                for r in res["result"]:
                    data["malware"].append(
                        {
                            "source": "HybridAnalysis",
                            "hash": r["sha256"],
                            "date": parse(r["analysis_start_time"]).astimezone(
                                pytz.utc
                            ),
                        }
                    )
        elif type == "hash":
            print("[+] Checking HybridAnalysis...")
            try:
                res = ha.search_hash(query)
            except HybridAnalysisFailed:
                pass
            else:
                for sample in res:
                    data["samples"].append(
                        {
                            "source": "HybridAnalysis",
                            "date": parse(sample["analysis_start_time"]).astimezone(
                                pytz.utc
                            ),
                            "url": "https://www.hybrid-analysis.com/sample/{}".format(
                                sample["sha256"]
                            ),
                            "infos": {
                                "Verdict": sample["verdict"],
                                "SubmitName": sample["submit_name"],
                                "Malware Family": sample["vx_family"],
                            },
                        }
                    )
                    if "domains" in sample:
                        for d in sample["domains"]:
                            data["network"].append(
                                {
                                    "source": "HybridAnalysis",
                                    "url": "https://www.hybrid-analysis.com/sample/{}".format(
                                        sample["sha256"]
                                    ),
                                    "host": d,
                                }
                            )
                    if "hosts" in sample:
                        for d in sample["hosts"]:
                            data["network"].append(
                                {
                                    "source": "HybridAnalysis",
                                    "url": "https://www.hybrid-analysis.com/sample/{}".format(
                                        sample["sha256"]
                                    ),
                                    "host": d,
                                }
                            )



#! /usr/bin/env python
from censys.common.exceptions import CensysRateLimitExceededException
from passivetotal.libs.enrichment import EnrichmentRequest
from virus_total_apis import PrivateApi, PublicApi

from harpoon.commands.base import Command
from harpoon.commands.censyscmd import CommandCensys
from harpoon.lib.utils import unbracket


class CommandSubdomains(Command):
    """
    # Subdomains plugin

    **Research subdomains for a domain**

    * Search for subdomains : `harpoon subdomains DOMAIN`

    """

    name = "subdomains"
    description = "Research subdomains of a domain"

    def add_arguments(self, parser):
        parser.add_argument("DOMAIN", help="Domain")
        parser.add_argument("--verbose", "-v", action="store_true", help="Verbose mode")
        parser.add_argument(
            "--source",
            "-s",
            choices=["all", "censys", "pt", "vt"],
            default="all",
            help="Source of the research",
        )
        self.parser = parser

    def censys_certs(self, domain, conf, verbose):
        censys_cmd = CommandCensys()
        print("[+] Searching through Censys certificates")
        subdomains = censys_cmd.get_subdomains(
            conf,
            domain,
            verbose,
        )
        return subdomains

    def pt(self, domain, conf, verbose):
        client = EnrichmentRequest(
            conf["PassiveTotal"]["username"], conf["PassiveTotal"]["key"]
        )
        print("[+] Searching subdomains in Passive Total")
        res = client.get_subdomains(query=domain)
        cleaned = []
        for sub in res["subdomains"]:
            cleaned.append(sub + "." + domain)

        return cleaned

    def vt(self, domain, conf, verbose):
        print("[+] Searching subdomains in Virus Total")
        if conf["VirusTotal"]["type"] == "public":
            vt = PublicApi(conf["VirusTotal"]["key"])
        else:
            vt = PrivateApi(conf["VirusTotal"]["key"])
        res = vt.get_domain_report(domain)

        if res["response_code"] == 204:
            print("VT quota exceeded!")
            return []
        else:
            return res["results"]["subdomains"]

    def prepare_data(self, subdomains=dict, data=dict, source=str):
        if len(subdomains) > 0:
            for domain in subdomains:
                data["subdomains"].append({"source": source, "domain": domain})

    def intel(self, type, query, data):
        if type == "domain":
            try:
                # subdomains = self.censys_certs(unbracket(query), self._confif_data, True)
                # self.prepare_data(subdomains, data, "Censys")
                pass

            except CensysRateLimitExceededException:
                print("Censys quota exceeded!")
            # subdomains = self.pt(unbracket(query), conf, True)
            # self.prepare_data(subdomains, data, "PassiveTotal")
            # subdomains = self.vt(unbracket(query), conf, True)
            # self.prepare_data(subdomains, data, "VirusTotal")

        else:
            pass

    def run(self, conf, args, plugins):
        if args.source == "all":
            # Search subdomains through a search in Censys certificates
            if plugins["censys"].test_config(conf):
                try:
                    subs = self.censys_certs(unbracket(args.DOMAIN), conf, args.verbose)
                    for sub in subs:
                        print(sub)
                except CensysRateLimitExceededException:
                    print("Quota exceeded!")
            if plugins["pt"].test_config(conf):
                subs = self.pt(unbracket(args.DOMAIN), conf, args.verbose)
                for sub in subs:
                    print(sub)
            if plugins["vt"].test_config(conf):
                subs = self.vt(unbracket(args.DOMAIN), conf, args.verbose)
                for sub in subs:
                    print(sub)

        elif args.source == "censys":
            if plugins["censys"].test_config(conf):
                try:
                    subs = self.censys_certs(unbracket(args.DOMAIN), conf, args.verbose)
                    for sub in subs:
                        print(sub)
                except CensysRateLimitExceededException:
                    print("Quota exceeded!")
            else:
                print("Please configure your Censys credentials")
        elif args.source == "pt":
            if plugins["pt"].test_config(conf):
                subs = self.pt(unbracket(args.DOMAIN), conf, args.verbose)
                for sub in subs:
                    print(sub)
            else:
                print("Please configure your Passive Total credentials")
        elif args.source == "vt":
            if plugins["vt"].test_config(conf):
                subs = self.vt(unbracket(args.DOMAIN), conf, args.verbose)
                for sub in subs:
                    print(sub)
            else:
                print("Please configure your VirusTotal credentials")



#! /usr/bin/env python
import json

import pypdns
import pytz

from harpoon.commands.base import Command
from harpoon.lib.utils import json_serial, unbracket


class CommandCircl(Command):
    """
    # Circl plugin

    **Query CIRCL passive DNS database (https://www.circl.lu/services/passive-dns/)**

    * Search for a domain : `harpoon circl DOMAIN`
    """

    name = "circl"
    description = "Request the CIRCL passive DNS database"
    config = {"Circl": ["user", "pass"]}

    def add_arguments(self, parser):
        parser.add_argument("DOMAIN", help="Domain")
        self.parser = parser

    def run(self, args, plugins):
        x = pypdns.PyPDNS(
            basic_auth=(
                self._config_data["Circl"]["user"],
                self._config_data["Circl"]["pass"],
            )
        )
        res = x.query(unbracket(args.DOMAIN))
        print(
            json.dumps(
                res,
                sort_keys=True,
                indent=4,
                separators=(",", ": "),
                default=json_serial,
            )
        )

    def intel_domain(self, query, data):
        print("[+] Downloading CIRCL passive DNS information....")
        x = pypdns.PyPDNS(
            basic_auth=(
                self._config_data["Circl"]["user"],
                self._config_data["Circl"]["pass"],
            )
        )
        res = x.query(query)
        for answer in res:
            data["passive_dns"].append(
                {
                    "ip": answer["rdata"],
                    "first": answer["time_first"].astimezone(pytz.utc),
                    "last": answer["time_last"].astimezone(pytz.utc),
                    "source": "CIRCL",
                }
            )



#! /usr/bin/env python3

import re

import requests


class UrlHausError(Exception):
    pass


class UrlHaus(object):
    """
    urlhaus.abuse.ch API Wrapper

    API documentation:
    * https://urlhaus-api.abuse.ch/#urlinfo
    * https://urlhaus.abuse.ch/api/#retrieve
    """

    def __init__(self, key):
        self.key = key
        self.base_url = "https://urlhaus-api.abuse.ch/v1/"
        self.ua = "Harpoon (https://github.com/Te-k/harpoon)"

    def _query(self, query):
        headers = ({"User-Agent": self.ua},)
        return requests.get(self.base_url + query, headers=headers)

    def launch_post_query(self, url, key, query):
        """
        Wrapper for POST queries
        """
        try:
            r = requests.post(
                url,
                headers={"User-Agent": self.ua},
                data={key: query},
            )
            if r.ok:
                return r.json()
            else:
                raise UrlHausError()
        except Exception as e:
            raise UrlHausError(e)

    def download_sample(self, url, query):
        """
        Wrapper for GET queries
        """
        try:
            r = requests.get(
                url,
                headers={"User-Agent": self.ua},
                data=query,
            )
            if r.ok:
                try:
                    return r.content
                except:  # noqa: E722
                    raise UrlHausError()
            else:
                return None
        except Exception as e:
            raise UrlHausError(e)

    def get_url(self, query):
        """
        Retrieve information about a given URL
        """
        url = "{}url/".format(self.base_url)
        return self.launch_post_query(url, "url", query)

    def get_host(self, query):
        """
        Retrieve information about a given host
        """

        url = "{}host/".format(self.base_url)
        return self.launch_post_query(url, "host", query)

    def get_payload(self, query):
        """
        Retrieve information about a md5 or sh256 payload
        """
        url = "{}payload/".format(self.base_url)
        # check if the hash is md5 or sha256
        if re.match(r"[A-Fa-f0-9]{64}", query.strip()):
            return self.launch_post_query(url, "sha256_hash", query.strip())
        elif re.match(r"[a-fA-F\d]{32}", query.strip()):
            return self.launch_post_query(url, "md5_hash", query.strip())
        else:
            raise UrlHausError()

    def get_tag(self, query):
        """
        Retrieve information about a tag
        """
        url = "{}tag/".format(self.base_url)
        return self.launch_post_query(url, "tag", query)

    def get_signature(self, query):
        """
        Retrieve information about a signature
        """
        url = "{}signature/".format(self.base_url)
        return self.launch_post_query(url, "signature", query)

    def get_sample(self, query):
        """
        Retrieve a malware sample
        """
        url = "{}/download".format(self.base_url)
        return self.download_sample(url, query)



#! /usr/bin/env python
import json

import pytz
from dateutil.parser import parse
from pysecuritytrails import SecurityTrails, SecurityTrailsError

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandSecurityTrails(Command):
    """
    # Security Trails

    Query https://securitytrails.com/

    * Get your quota : `harpoon securitytrails quota`
    * Get historical whois info on a domain : `harpoon securitytrails whois DOMAIN`
    * Get Passive DNS data on a domain : `harpoon securitytrails domain DOMAIN`
    * Get Passive DNS data on an IP : `harpoon securitytrails ip IP` (IPv4 only)
    * Get subdomains of a domain : `harpoon subdomains DOMAIN`

    """

    name = "securitytrails"
    description = "Requests SecurityTrails database"
    config = {"SecurityTrails": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("whois", help="Request whois info")
        parser_a.add_argument("DOMAIN", help="DOMAIN to be queried")
        parser_a.set_defaults(subcommand="whois")
        parser_b = subparsers.add_parser("quota", help="Check quota")
        parser_b.set_defaults(subcommand="quota")
        parser_c = subparsers.add_parser("domain", help="Passive DNS info")
        parser_c.add_argument("DOMAIN", help="DOMAIN to be queried")
        parser_c.set_defaults(subcommand="domain")
        parser_d = subparsers.add_parser("ip", help="Passive DNS info")
        parser_d.add_argument("IP", help="Search for passive DNS on an IP")
        parser_d.set_defaults(subcommand="ip")
        parser_e = subparsers.add_parser("subdomains", help="Get subdomains")
        parser_e.add_argument("DOMAIN", help="Domain to be queried")
        parser_e.set_defaults(subcommand="subdomains")
        self.parser = parser

    def run(self, args, plugins):
        client = SecurityTrails(self._config_data["SecurityTrails"]["key"])
        if "subcommand" in args:
            if args.subcommand == "whois":
                res = client.domain_history_whois(unbracket(args.DOMAIN))
                print(json.dumps(res["result"]["items"], sort_keys=False, indent=4))
            elif args.subcommand == "quota":
                res = client.usage()
                print(
                    "Quota : {} / {}".format(
                        res["current_monthly_usage"], res["allowed_monthly_usage"]
                    )
                )
            elif args.subcommand == "domain":
                res = client.domain_history_dns(unbracket(args.DOMAIN))
                print(json.dumps(res["records"], sort_keys=False, indent=4))
            elif args.subcommand == "ip":
                res = client.domain_search(
                    {"ipv4": unbracket(args.IP)}, include_ips=True
                )
                print(json.dumps(res["records"], sort_keys=False, indent=4))
            elif args.subcommand == "subdomains":
                res = client.domain_subdomains(unbracket(args.DOMAIN))
                for d in res["subdomains"]:
                    print(d + "." + args.DOMAIN)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_domain(self, query, data):
        client = SecurityTrails(self._config_data["SecurityTrails"]["key"])
        print("[+] Checking SecurityTrails...")
        try:
            res = client.domain_history_dns(query)
            for r in res["records"]:
                for ip in r["values"]:
                    data["passive_dns"].append(
                        {
                            "ip": ip["ip"],
                            "source": "SecurityTrails",
                            "first": parse(r["first_seen"]).astimezone(pytz.utc),
                            "last": parse(r["last_seen"]).astimezone(pytz.utc),
                        }
                    )
        except SecurityTrailsError:
            print("Security Trail request failed")



import requests
from requests.auth import HTTPBasicAuth


class CertSpotterError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class CertSpotter(object):
    def __init__(self, key=None):
        self.key = key
        self.base_url = "https://api.certspotter.com/"

    @property
    def authenticated(self):
        return (self.key is not None)

    def _get(self, query, params={}):
        if self.authenticated:
            r = requests.get(
                self.base_url + query,
                params=params,
                auth=HTTPBasicAuth(self.key, "")
            )
        else:
            r = requests.get(
                self.base_url + query,
                params=params
            )
        if r.status_code == 200:
            return r.json()
        else:
            raise CertSpotterError("Invalid HTTP status code %i" % r.status_code)

    def search(self, domain, include_subdomains=False):
        return self._get(
                "v1/issuances",
                params={
                    "domain": domain,
                    "expand": ["dns_names", "issuer", "cert"],
                    "include_subdomains": include_subdomains})



#! /usr/bin/env python
import os
import subprocess

from harpoon.commands.base import Command


class CommandUpdate(Command):
    """
    # Update command

    **Update information used by Harpoon**

    * `harpoon update`

    Downloads several files in harpoon config folder (.config/harpoon)
    """

    name = "update"
    description = "Update Harpoon data"

    def add_arguments(self, parser):
        pass

    def run(self, args, plugins):
        configdir = os.path.join(os.path.expanduser("~"), ".config/harpoon")
        if not os.path.isdir(configdir):
            os.makedirs(configdir)
        print("Updating all plugins data:")
        for p in plugins:
            if plugins[p].update_needed:
                print("Updating plugin %s" % p)
                plugins[p].update()
        print("Updating GeoIP database")
        print("sudo geoipupdate")
        a = subprocess.run(["sudo", "geoipupdate"])
        if a.returncode != 0:
            print("Impossible to launch geoipupdate")
            print(
                "Please make sure you have the package installed and correctly configured"
            )
            print("See https://github.com/Te-k/harpoon/wiki/Installation")



#! /usr/bin/env python
import requests
from pypermacc import Permacc, PermaccError

from harpoon.commands.base import Command
from harpoon.lib.archiveis import ArchiveIs
from harpoon.lib.archiveorg import ArchiveOrg
from harpoon.lib.utils import unbracket


class CommandSave(Command):
    """
    # Save plugin

    **Save an URL in cache platforms (archive.is, archive.org, perma.cc)**

    Commands:
    * Save an url: `harpoon save URL`
    """

    name = "save"
    description = "Save a webpage in cache platforms"
    config = {}

    def add_arguments(self, parser):
        parser.add_argument("URL", help="URL to save")
        self.parser = parser

    def run(self, args, plugins):
        print("Saving in cache platforms:")
        # Archive.is
        try:
            ai_url = ArchiveIs.capture(unbracket(args.URL))
        except requests.exceptions.TooManyRedirects:
            print("Impossible to save in Archive.is")
        else:
            print("Archive.is: %s" % ai_url)

        # Web Archive
        try:
            ao_url = ArchiveOrg.capture(unbracket(args.URL))
        except KeyError:
            print("Impossible to save in Web Archive")
        else:
            print("Web Archive: %s" % ao_url)

        # Perma.cc
        if "Permacc" in self._config_data and "key" in self._config_data["Permacc"]:
            pc = Permacc(self._config_data["Permacc"]["key"])
            try:
                saved = pc.archive_create(unbracket(args.URL))
            except PermaccError:
                print("Impossible to save in Permacc")
            else:
                print("Permacc: https://perma.cc/%s" % saved["guid"])



#! /usr/bin/env python
import json

from harpoon.commands.base import Command
from harpoon.lib.hibp import HIBP, HibpNotFound


class CommandHibp(Command):
    """
    # Have I Been Pwned

    **Requests Have I Been Pwned API https://haveibeenpwned.com/**

    * Search for an email: `harpoon hibp admin@example.org`
    * Search for leaks in pasts: `harpoon bibp -p admin@example.org`
    * See raw JSON output: `harpoon hibp -j admin@example.org`
    """

    name = "hibp"
    description = "Request Have I Been Pwned API (https://haveibeenpwned.com/)"
    config = {"hibp": ["key"]}

    def add_arguments(self, parser):
        parser.add_argument("EMAIL", help="Email to check")
        parser.add_argument(
            "--pastes",
            "-p",
            help="Check pastes instead of breaches",
            action="store_true",
        )
        parser.add_argument(
            "--json", "-j", action="store_true", help="Check pastes instead of breaches"
        )
        self.parser = parser

    def run(self, args, plugins):
        h = HIBP(self._config_data["hibp"]["key"])
        if args.pastes:
            try:
                res = h.get_pastes(args.EMAIL)
            except HibpNotFound:
                print("Account not found in the HIBP database")
            else:
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    print("Account found in %i pastes:" % len(res))
                    for r in res:
                        print(
                            '[+] %s: %s %s "%s"'
                            % (r["Date"], r["Source"], r["Id"], r["Title"])
                        )
        else:
            try:
                res = h.get_breaches_account(args.EMAIL)
            except HibpNotFound:
                print("Account not found in the HIBP database")
            else:
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    print("Account found in %i breaches\n" % len(res))
                    for r in res:
                        print(r["Name"])



import json

import requests
from simplejson.errors import JSONDecodeError


class DNSDBError(Exception):
    def __init__(self, message):
        self.message = message
        Exception.__init__(self, message)


class DNSDBInvalidParameters(Exception):
    pass


class DnsDB(object):
    """
    Implementation of Farsight DNSDB API
    See https://docs.farsightsecurity.com/dnsdb/dnsdb-apiv2/
    """
    def __init__(self, token):
        self.token = token
        self.base_url = "https://api.dnsdb.info/"

    def _get(self, query, params={}):
        headers = {
            "User-Agent": "harpoon (https://github.com/Te-k/harpoon/)",
            "X-API-Key": self.token
        }
        r = requests.get(self.base_url + query, headers=headers)
        if r.status_code != 200:
            raise DNSDBError('Invalid HTTP code %i' % r.status_code)
        try:
            return r.json()
        except JSONDecodeError:
            # JSONL
            res = []
            for line in r.text.split("\n"):
                if line.strip() == "":
                    continue
                dd = json.loads(line)
                if "obj" in dd:
                    res.append(dd["obj"])
        return res

    def ping(self):
        return self._get("dnsdb/v2/ping")

    def rrset_lookup(self, value, rrtype="ANY"):
        return self._get("dnsdb/v2/lookup/rrset/name/{}/{}".format(value, rrtype))

    def rdata_lookup(self, value, type="name", rrtype="ANY"):
        if type not in ["name", "ip", "raw"]:
            raise DNSDBInvalidParameters("Invalid type parameter")
        return self._get("dnsdb/v2/lookup/rdata/{}/{}/{}".format(type, value, rrtype))



#! /usr/bin/env python
import json
import os
import sys

from harpoon.commands.base import Command
from harpoon.lib.cybercure import CyberCure, CyberCureError
from harpoon.lib.utils import unbracket


class CommandCyberCure(Command):
    """
    # cybercure.ai plugin

    **Check if intelligence on an IP exists**

    Query cybercure API:
    ```
    harpoon cybercure ip 184.186.250.211
    {
    "exists": true,
    "indicator": "184.186.250.211",
    "status": 1,
    "visual": "http://www.cybercure.ai/intel/ip/184.186.250.211"
    }
    ```

    """

    name = "cybercure"
    description = "Search cybercure.ai intelligence database for specific indicators."

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "ip",
            help="Returns a response whether an\
                indicator exists in cybercure.ai database",
        )
        parser_a.add_argument("IP", help="IP address")
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser("file", help="Information on a list of IPs")
        parser_b.add_argument("FILE", help="Filename")
        parser_b.set_defaults(subcommand="file")
        self.parser = parser

    def run(self, args, plugins):
        cybercure = CyberCure(token="reserved_for_future")
        if "subcommand" in args:
            if args.subcommand == "ip":
                try:
                    infos = cybercure.get_infos(unbracket(args.IP))
                except CyberCureError:
                    print("Invalid request")
                else:
                    print(
                        json.dumps(
                            infos, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            elif args.subcommand == "file":
                if os.path.isfile(args.FILE):
                    with open(args.FILE) as f:
                        data = f.read().split("\n")
                    print("IP;Exists;Details")
                    for d in data:
                        if d.strip() == "":
                            continue
                        ip = unbracket(d.strip())
                        try:
                            infos = cybercure.get_infos(ip)
                        except CyberCureError:
                            print("%s;;" % ip)
                        else:
                            print(
                                "{};{};{}".format(
                                    ip,
                                    infos["exists"],
                                    infos["visual"] if "visual" in infos else "",
                                )
                            )
                else:
                    print("This file does not exist")
                    sys.exit(1)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests


class IP2LocationioError(Exception):
    def __init__(self, message):
        self.message = message
        Exception.__init__(self, message)


class IP2Locationio(object):
    def __init__(self, token):
        self.token = token
        self.base_url = 'https://api.ip2location.io/'

    def get_infos(self, ip):
        r = requests.get(self.base_url, params={'key': self.token, 'ip': ip})
        if r.status_code != 200:
            raise IP2LocationioError('Invalid HTTP code %i' % r.status_code)
        return r.json()



#! /usr/bin/env python
import os
import subprocess
import sys
from shutil import copyfile

from harpoon.commands.base import Command


class CommandConfig(Command):
    """
    # Configuration command

    **Help managing Harpoon configuration**

    * Create and update configuration file: `harpoon config`
    * Test plugins' configuration: `harpoon config -c`
    * Download or update databases: `harpoon config -u`

    Configuration file is stored in `~/.harpoon/config`
    """

    name = "config"
    description = "Configure Harpoon"

    def add_arguments(self, parser):
        parser.add_argument(
            "--show", "-s", action="store_true", help="Show harpoon configuration"
        )
        parser.add_argument(
            "--check", "-c", action="store_true", help="Config harpoon configuration"
        )

    def run(self, args, plugins):
        configdir = os.path.join(os.path.expanduser("~"), ".config/harpoon")
        configpath = os.path.join(os.path.expanduser("~"), ".config/harpoon/config")
        if args.show:
            if not os.path.isfile(configpath):
                print("Config file does not exist, use harpoon config to create it")
                sys.exit(1)
            else:
                with open(configpath, "r") as f:
                    print(f.read())
        elif args.check:
            print("Configuration check:")
            for p in plugins:
                if plugins[p].config_needed:
                    if plugins[p].test_config():
                        if len(p) < 7:
                            print("-%s\t\t -> OK" % p)
                        else:
                            print("-%s\t -> OK" % p)
                    else:
                        if len(p) < 7:
                            print("-%s\t\t -> FAILED" % p)
                        else:
                            print("-%s\t -> FAILED" % p)
                else:
                    if len(p) < 7:
                        print("-%s\t\t -> OK" % p)
                    else:
                        print("-%s\t -> OK" % p)
        else:
            if not os.path.isdir(configdir):
                os.makedirs(configdir)
            if not os.path.isfile(configpath):
                origpath = os.path.join(
                    os.path.realpath(__file__)[:-18], "data/example.conf"
                )
                copyfile(origpath, configpath)
            subprocess.call(
                os.environ.get("EDITOR", "vi") + " " + configpath, shell=True
            )



#! /usr/bin/env python
import json
import os
import sys
import time

from harpoon.commands.base import Command
from harpoon.lib.ipinfo import IPInfo, IPInfoError
from harpoon.lib.utils import unbracket


class CommandIPInfo(Command):
    """
    # ipinfo.io plugin

    **Query ipinfo.io API**

    * Get info on an IP : `harpoon ipinfo ip IP`
    * Get infos on a list of IPs in a file : `harpoon ipinfo file FILE`
    """

    name = "ipinfo"
    description = "Request ipinfo.io information"
    config = {"IPInfo": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("ip", help="Information on an IP")
        parser_a.add_argument("IP", help="IP address")
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser("file", help="Information on a list of IPs")
        parser_b.add_argument("FILE", help="Filename")
        parser_b.add_argument(
            "--delay",
            "-d",
            type=int,
            default=1,
            help="Delay between two queries in seconds",
        )
        parser_b.set_defaults(subcommand="file")
        self.parser = parser

    def run(self, args, plugins):
        ipinfo = IPInfo(token=self._config_data["IPInfo"]["token"])
        if "subcommand" in args:
            if args.subcommand == "ip":
                try:
                    infos = ipinfo.get_infos(unbracket(args.IP))
                except IPInfoError:
                    print("Invalid request")
                else:
                    print(
                        json.dumps(
                            infos, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            elif args.subcommand == "file":
                if os.path.isfile(args.FILE):
                    with open(args.FILE) as f:
                        data = f.read().split("\n")
                    print(
                        "IP;Hostname;City;Region;Country;Location;"
                        + "Company Name;Company Domain;Company Type;ASN;"
                        + "AS Name;AS Domain;AS Route;AS Type"
                    )
                    for d in data:
                        if d.strip() == "":
                            continue
                        ip = unbracket(d.strip())
                        try:
                            infos = ipinfo.get_infos(ip)
                        except IPInfoError:
                            print("%s;;;;;;;;;;;;;" % ip)
                        else:
                            if "company" in infos and "asn" in infos:
                                print(
                                    "%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s"
                                    % (
                                        ip,
                                        infos["hostname"]
                                        if "hostname" in infos
                                        else "",
                                        infos["city"] if "city" in infos else "",
                                        infos["region"] if "region" in infos else "",
                                        infos["country"] if "country" in infos else "",
                                        infos["loc"] if "loc" in infos else "",
                                        infos["company"]["name"]
                                        if "company" in infos
                                        else "",
                                        infos["company"]["domain"]
                                        if "company" in infos
                                        else "",
                                        infos["company"]["type"]
                                        if "company" in infos
                                        else "",
                                        infos["asn"]["asn"]
                                        if "asn" in infos["asn"]
                                        else "",
                                        infos["asn"]["name"]
                                        if "name" in infos["asn"]
                                        else "",
                                        infos["asn"]["domain"]
                                        if "domain" in infos["asn"]
                                        else "",
                                        infos["asn"]["route"]
                                        if "route" in infos["asn"]
                                        else "",
                                        infos["asn"]["type"]
                                        if "type" in infos["asn"]
                                        else "",
                                    )
                                )
                            elif "company" in infos and "asn" not in infos:
                                print(
                                    "%s;%s;%s;%s;%s;%s;%s;%s;%s;;;;;"
                                    % (
                                        ip,
                                        infos["hostname"]
                                        if "hostname" in infos
                                        else "",
                                        infos["city"],
                                        infos["region"],
                                        infos["country"],
                                        infos["loc"],
                                        infos["company"]["name"],
                                        infos["company"]["domain"],
                                        infos["company"]["type"],
                                    )
                                )
                            elif "asn" in infos and "company" not in infos:
                                print(
                                    "%s;%s;%s;%s;%s;%s;;;;%s;%s;%s;%s;%s"
                                    % (
                                        ip,
                                        infos["hostname"]
                                        if "hostname" in infos
                                        else "",
                                        infos["city"],
                                        infos["region"],
                                        infos["country"],
                                        infos["loc"],
                                        infos["asn"]["asn"],
                                        infos["asn"]["name"],
                                        infos["asn"]["domain"],
                                        infos["asn"]["route"],
                                        infos["asn"]["type"],
                                    )
                                )
                            else:
                                print(
                                    "%s;%s;%s;%s;%s;%s;;;;%s;%s;;;"
                                    % (
                                        ip,
                                        infos["hostname"]
                                        if "hostname" in infos
                                        else "",
                                        infos["city"],
                                        infos["region"],
                                        infos["country"],
                                        infos["loc"],
                                        infos["org"].split(" ")[0],
                                        " ".join(infos["org"].split(" ")[1:]),
                                    )
                                )
                        time.sleep(args.delay)

                else:
                    print("This file does not exist")
                    sys.exit(1)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import json
import sys

from harpoon.commands.base import Command
from harpoon.lib.certspotter import CertSpotter, CertSpotterError
from harpoon.lib.utils import json_serial, unbracket


class CommandCertSpotter(Command):
    """
    # Cert Spotter Command

    Search in Certificate Transparency database Cert Spotter https://sslmate.com/certspotter.
    Only current certificates can be searched without paid plan

    * Search certificates for a domain : `harpoon certspotter search DOMAIN`
    * Search certificates for a domain and its subdomains : `harpoon certspotter search -s DOMAIN`
    """

    name = "certspotter"
    description = "Get certificates from https://sslmate.com/certspotter"
    config = None

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "search", help="Search certificates for a domain"
        )
        parser_a.add_argument("DOMAIN", help="domain")
        parser_a.add_argument(
            "--subdomains",
            "-s",
            help="Search for the domain and its subdomains",
            action="store_true",
        )
        parser_a.set_defaults(subcommand="search")
        self.parser = parser

    def run(self, args, plugins):
        try:
            cs = CertSpotter(self._config_data["CertSpotter"]["key"])
        except KeyError:
            cs = CertSpotter()

        if "subcommand" in args:
            if args.subcommand == "search":
                try:
                    res = cs.search(
                        unbracket(args.DOMAIN), include_subdomains=args.subdomains
                    )
                except CertSpotterError:
                    print(
                        "Error with the API, likely because you need a paid plan to search expired certs."
                    )
                    print("Check censys or crtsh plugins instead")
                    sys.exit(1)
                else:
                    print(
                        json.dumps(
                            res,
                            sort_keys=True,
                            indent=4,
                            separators=(",", ": "),
                            default=json_serial,
                        )
                    )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import re
from datetime import date, datetime
from urllib.parse import urlparse

from IPy import IP


def unbracket(domain):
    """Remove protective bracket from a domain"""
    return domain.replace("[.]", ".")


def bracket(domain):
    """Add protective bracket to a domain"""
    last_dot = domain.rfind(".")
    return domain[:last_dot] + "[.]" + domain[last_dot+1:]


def json_serial(obj):
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    raise TypeError("Type %s not serializable" % type(obj))


def same_url(url1, url2):
    """
    Check for minor differences between url1 and url2, return True if they are the same
    Currently only consider extra www. in domain, https/http and extra fragment
    """
    if url1 == url2:
        return True
    # Dirty hacks
    if not url1.startswith('http'):
        url1 = 'http://' + url1
    if not url2.startswith('http'):
        url2 = 'http://' + url2
    if not url1.endswith('/'):
        url1 += '/'
    if not url2.endswith('/'):
        url2 += '/'
    purl2 = urlparse(url2)
    purl1 = urlparse(url1)

    if purl1.path == purl2.path and purl1.params == purl2.params and \
            purl1.query == purl2.query:
        if purl1.netloc == purl2.netloc:
            return True
        else:
            if ("www." + purl1.netloc) == purl2.netloc:
                return True
            if ("www." + purl2.netloc) == purl1.netloc:
                return True
    return False


def typeguess(indicator):
    """
    Guess the type of the indicator
    returns string in "IPv4", "IPv6", "md5", "sha1", "sha256", "domain"
    """
    if re.match(r"^\w{32}$", indicator):
        return "md5"
    elif re.match(r"^\w{40}$", indicator):
        return "sha1"
    elif re.match(r"^\w{64}$", indicator):
        return "sha256"
    else:
        try:
            i = IP(indicator)
            if i.version() == 4:
                return "IPv4"
            else:
                return "IPv6"
        except ValueError:
            return "domain"


def is_ip(target):
    """
    Test if a string is an IP address
    """
    if isinstance(target, str):
        try:
            IP(target)
            return True
        except ValueError:
            return False
    else:
        return False


def ts_to_str(ts):
    return datetime.fromtimestamp(ts).isoformat()


def is_sha256(string):
    return re.match(r"^\w{64}$", string) is not None



import requests

from harpoon.lib.utils import is_sha256


class HybridAnalysisFailed(Exception):
    pass


class HybridAnalysisInvalidQuery(Exception):
    pass


class HybridAnalysis(object):
    def __init__(self, key):
        self.key = key
        self.base_url = "https://www.hybrid-analysis.com/api/v2/"
        # Mandatory for Hybrid Analysis (weird)
        self.ua = "Falcon Sandbox"

    def _get(self, path):
        """
        Query the API with the given path
        """
        headers = {
            "User-Agent": self.ua,
            "api-key": self.key
        }
        r = requests.get(
            self.base_url + path,
            headers=headers
        )
        if r.status_code != 200:
            raise HybridAnalysisFailed()
        else:
            return r.json()

    def _post(self, path, data={}):
        """
        Query the API with the given path
        """
        headers = {
            "User-Agent": self.ua,
            "api-key": self.key
        }
        r = requests.post(
            self.base_url + path,
            headers=headers,
            data=data
        )
        if r.status_code != 200:
            raise HybridAnalysisFailed()
        else:
            return r.json()

    # --------------------------- search --------------------------------------
    def search_hash(self, hash):
        """
        Summary for a given hash
        hash should be md5, sha1 or SHA256
        """
        return self._post(
            "search/hash",
            data={"hash": hash}
        )

    def search_terms(self, data):
        """
        Search the database using search terms
        Accepted keywords :
        filename
        filetype
        filetype_desc
        env_id
        country
        verdict
        av_detect
        vx_family
        date_from
        date_to
        port
        host
        domain
        url
        similar_to
        context
        imp_hash
        ssdeep
        authentihash
        uses_tactics
        uses_tecnique
        """
        terms = [
            "filename", "filetype", "filetype_desc", "env_id", " country",
            "verdict", "av_detect", "vx_family", "date_from", "date_to",
            "port", "host", "domain", "url", "similar_to", "context",
            "imp_hash", "ssdeep", "authentihash", "uses_tactics", "uses_tecnique"]
        for d in data:
            if d not in terms:
                raise HybridAnalysisInvalidQuery("{} is not a valid search term".format(d))

        return self._post("search/terms", data)

        # ------------------------------- Overview --------------------------------
    def overview_hash(self, hash):
        """
        Return overview of a hash
        /overview/{sha256}
        """
        if not is_sha256(hash):
            raise HybridAnalysisInvalidQuery("Invalid sha256 format")
        return self._get('overview/' + hash)

    def overview_summary(self, hash):
        """
        Returns overview for a hash
        /overview/{sha256}/summary
        """
        if not is_sha256(hash):
            raise HybridAnalysisInvalidQuery("Invalid sha256 format")
        return self._get('overview/' + hash + "/summary")

    def overview_sample(self, hash):
        """
        Download a sample
        """
        if not is_sha256(hash):
            raise HybridAnalysisInvalidQuery("Invalid sha256 format")
        headers = {
            "User-Agent": self.ua,
            "api-key": self.key
        }
        r = requests.get(
            self.base_url + "overview/" + hash + "/sample",
            headers=headers,
        )
        if r.status_code != 200:
            raise HybridAnalysisFailed()
        else:
            return r.content



#! /usr/bin/env python
import json
import re

import pytz
from dateutil.parser import parse
from zetalytics import Zetalytics

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandZetalytics(Command):
    """
    # Zetalytics

    Query Zetalytics API https://zetalytics.com/
    """

    name = "zetalytics"
    description = "Search in Zetalytics database"
    config = {"Zetalytics": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "cname2qname",
            help="Search passive dns by CNAME value (target of CNAME record)",
        )
        parser_a.add_argument("CNAME", help="CNAME value to query (matches subdomains)")
        parser_a.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_a.set_defaults(subcommand="cname2qname")
        parser_b = subparsers.add_parser(
            "domain2aaaa", help="Search passive dns by domain for AAAA (IPv6) records"
        )
        parser_b.add_argument("DOMAIN", help="Domain")
        parser_b.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_b.set_defaults(subcommand="domain2aaaa")
        parser_c = subparsers.add_parser(
            "domain2cname", help="Search passive dns by domain for CNAME records"
        )
        parser_c.add_argument("DOMAIN", help="Domain")
        parser_c.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_c.set_defaults(subcommand="domain2cname")
        parser_d = subparsers.add_parser(
            "domain2d8s", help="Search historical d8s records and/or live d8s"
        )
        parser_d.add_argument("DOMAIN", help="Domain")
        parser_d.set_defaults(subcommand="domain2d8s")
        parser_e = subparsers.add_parser(
            "domain2ip", help="Search passive dns by domain for A (IPv4) records"
        )
        parser_e.add_argument("DOMAIN", help="Domain")
        parser_e.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_e.set_defaults(subcommand="domain2ip")
        parser_f = subparsers.add_parser(
            "domain2malwaredns", help="Search malware dns by domain"
        )
        parser_f.add_argument("DOMAIN", help="Domain")
        parser_f.set_defaults(subcommand="domain2malwaredns")
        parser_g = subparsers.add_parser(
            "domain2malwarehttp", help="Search malware http by domain"
        )
        parser_g.add_argument("DOMAIN", help="Domain")
        parser_g.set_defaults(subcommand="domain2malwarehttp")
        parser_h = subparsers.add_parser(
            "domain2mx", help="Search passive dns by domain for MX records"
        )
        parser_h.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_h.add_argument("DOMAIN", help="Domain")
        parser_h.set_defaults(subcommand="domain2mx")
        parser_i = subparsers.add_parser(
            "domain2ns", help="Search passive dns by domain for NS records"
        )
        parser_i.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_i.add_argument("DOMAIN", help="Domain")
        parser_i.set_defaults(subcommand="domain2ns")
        parser_j = subparsers.add_parser(
            "domain2nsglue", help="Search name server glue (IP) records by domain name."
        )
        parser_j.add_argument("DOMAIN", help="Domain")
        parser_j.set_defaults(subcommand="domain2nsglue")
        parser_k = subparsers.add_parser(
            "domain2ptr", help="Search passive dns by domain for PTR records"
        )
        parser_k.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_k.add_argument("DOMAIN", help="Domain")
        parser_k.set_defaults(subcommand="domain2ptr")
        parser_l = subparsers.add_parser(
            "domain2txt", help="Search passive dns by domain for TXT records"
        )
        parser_l.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_l.add_argument("DOMAIN", help="Domain")
        parser_l.set_defaults(subcommand="domain2txt")
        parser_m = subparsers.add_parser(
            "domain2whois", help="Search historical whois records"
        )
        parser_m.add_argument("DOMAIN", help="Domain")
        parser_m.set_defaults(subcommand="domain2whois")
        parser_n = subparsers.add_parser(
            "email_address",
            help="Search for domains sharing a registration email address or SOA email from passive",
        )
        parser_n.add_argument("EMAIL", help="Email address")
        parser_n.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_n.set_defaults(subcommand="email_address")
        parser_o = subparsers.add_parser(
            "email_domain",
            help="Search for domains sharing a registration email address domain",
        )
        parser_o.add_argument("EMAIL", help="Email address")
        parser_o.set_defaults(subcommand="email_domain")
        parser_p = subparsers.add_parser(
            "email_user",
            help="Search for domains sharing a registration email address or SOA email from passive",
        )
        parser_p.add_argument("EMAIL", help="Email address")
        parser_p.set_defaults(subcommand="email_user")
        parser_q = subparsers.add_parser(
            "hash2malwaredns", help="Search malware dns by md5 hash"
        )
        parser_q.add_argument("HASH", help="Md5 hash")
        parser_q.set_defaults(subcommand="hash2malwaredns")
        parser_r = subparsers.add_parser(
            "hash2malwarehttp", help="Search malware http by md5 hash"
        )
        parser_r.add_argument("HASH", help="Md5 hash")
        parser_r.set_defaults(subcommand="hash2malwarehttp")
        parser_s = subparsers.add_parser(
            "hostname",
            help="Search passive dns by hostname for mixed resource record types",
        )
        parser_s.add_argument("DOMAIN", help="Domain name")
        parser_s.set_defaults(subcommand="hostname")
        parser_t = subparsers.add_parser(
            "ip", help="Search passive dns by IP, CIDR, or Range (v6 compatible)"
        )
        parser_t.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_t.add_argument("IP", help="IP or CIDR or IP range")
        parser_t.set_defaults(subcommand="ip")
        parser_u = subparsers.add_parser(
            "ip2malwaredns", help="Search malware dns by IP"
        )
        parser_u.add_argument("IP", help="IP address or CIDR or range IP")
        parser_u.set_defaults(subcommand="ip2malwaredns")
        parser_v = subparsers.add_parser(
            "ip2malwarehttp",
            help="Search malware http by IP/CIDR for http://x.x.x.x/ (not the IP a hostname resolved to)",
        )
        parser_v.add_argument("IP", help="IP address or CIDR or range IP")
        parser_v.set_defaults(subcommand="ip2malwarehttp")
        parser_w = subparsers.add_parser(
            "ip2nsglue",
            help="Search name server glue (IP) records by IP, CIDR, or Range (v6 compatible)",
        )
        parser_w.add_argument("IP", help="IP address or CIDR or range IP")
        parser_w.set_defaults(subcommand="ip2nsglue")
        parser_x = subparsers.add_parser(
            "mx2domain",
            help="Search passive dns by MX domain for any domain served by the MX domain",
        )
        parser_x.add_argument("MX", help="MX server")
        parser_x.set_defaults(subcommand="mx2domain")
        parser_y = subparsers.add_parser(
            "ns2domain",
            help="Search current zone files and passive DNS for domains served by nameserver",
        )
        parser_y.add_argument("NS", help="NS server")
        parser_y.set_defaults(subcommand="ns2domain")
        parser_z = subparsers.add_parser(
            "subdomains",
            help="Search passive dns by domain for a list of subdomains from any record type",
        )
        parser_z.add_argument("DOMAIN", help="Domain name")
        parser_z.set_defaults(subcommand="subdomains")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            zeta = Zetalytics(token=self._config_data["Zetalytics"]["token"])
            if args.subcommand == "cname2qname":
                res = zeta.cname2qname(q=unbracket(args.CNAME))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Not found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12}{:35} {:20} {}".format(
                                    r["last_seen"] if "last_seen" in r else "",
                                    r["qname"],
                                    r["domain"],
                                    r["value"],
                                )
                            )
            elif args.subcommand == "domain2aaaa":
                res = zeta.domain2aaaa(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["last_seen"] if "last_seen" in r else "",
                                    r["qname"],
                                    r["value"],
                                )
                            )
            elif args.subcommand == "domain2cname":
                res = zeta.domain2cname(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["last_seen"] if "last_seen" in r else "",
                                    r["qname"],
                                    r["value"],
                                )
                            )
            elif args.subcommand == "domain2d8s":
                res = zeta.domain2d8s(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "domain2ip":
                res = zeta.domain2ip(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["last_seen"] if "last_seen" in r else "",
                                    r["qname"],
                                    r["value"],
                                )
                            )
            elif args.subcommand == "domain2malwaredns":
                res = zeta.domain2malwaredns(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "domain2malwarehttp":
                res = zeta.domain2malwarehttp(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "domain2mx":
                res = zeta.domain2mx(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["date"], r["qname"], r["value"]
                                )
                            )
            elif args.subcommand == "domain2ns":
                res = zeta.domain2ns(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["date"], r["qname"], r["value"]
                                )
                            )
            elif args.subcommand == "domain2nsglue":
                res = zeta.domain2nsglue(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "domain2ptr":
                res = zeta.domain2ptr(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["date"], r["qname"], r["value"]
                                )
                            )
            elif args.subcommand == "domain2txt":
                res = zeta.domain2txt(q=unbracket(args.DOMAIN))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in res["results"]:
                            print(
                                "{:12} {:30} {}".format(
                                    r["date"], r["qname"], r["value"]
                                )
                            )
            elif args.subcommand == "domain2whois":
                res = zeta.domain2whois(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "email_address":
                res = zeta.email_address(q=unbracket(args.EMAIL))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Nothing found")
                    else:
                        for r in sorted(res["results"], key=lambda x: x["last_ts"]):
                            print(
                                "{} {} - {:30} - {}".format(
                                    r["first_ts"][:10],
                                    r["last_ts"][:10],
                                    r["d"],
                                    r["emails"][0]["addr"],
                                )
                            )
            elif args.subcommand == "email_domain":
                res = zeta.email_domain(q=unbracket(args.EMAIL))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "email_user":
                res = zeta.email_user(q=unbracket(args.EMAIL))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "hash2malwaredns":
                if not re.fullmatch(r"[a-fA-F\d]{32}", args.HASH):
                    print("Zetalytics only accept md5 hashes")
                else:
                    res = zeta.hash2malwaredns(q=args.HASH)
                    print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "hash2malwarehttp":
                if not re.fullmatch(r"[a-fA-F\d]{32}", args.HASH):
                    print("Zetalytics only accept md5 hashes")
                else:
                    res = zeta.hash2malwarehttp(q=args.HASH)
                    print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "hostname":
                res = zeta.hostname(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ip":
                res = zeta.ip(q=unbracket(args.IP))
                if args.json:
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    if res["total"] == 0:
                        print("Not found")
                    else:
                        for r in res["results"]:
                            print(
                                "{} - {} - {}".format(
                                    r["date"],
                                    r["last_seen"] if "last_seen" in r else "",
                                    r["qname"],
                                )
                            )
            elif args.subcommand == "ip2malwaredns":
                res = zeta.ip2malwaredns(q=unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ip2malwarehttp":
                res = zeta.ip2malwarehttp(q=unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ip2nsglue":
                res = zeta.ip2nsglue(q=unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "mx2domain":
                res = zeta.mx2domain(q=unbracket(args.MX))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ns2domain":
                res = zeta.ns2domain(q=unbracket(args.NS))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "subdomains":
                res = zeta.subdomains(q=unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        # TODO: support emails in here
        zeta = Zetalytics(token=self._config_data["Zetalytics"]["token"])
        if type == "domain":
            print("[+] Checking Zetalytics...")
            res = zeta.domain2ip(q=query)
            if "results" in res:
                for domain in res["results"]:
                    if domain["qname"] == query:
                        data["passive_dns"].append(
                            {
                                "ip": domain["value"],
                                "source": "Zetalytics",
                                "first": parse(domain["date"]).astimezone(pytz.utc),
                                "last": parse(domain["last_seen"]).astimezone(pytz.utc)
                                if "last_seen" in domain
                                else None,
                            }
                        )
                    # else:
                    # data["subdomains"].append(domain["qname"])
        elif type == "ip":
            print("[+] Checking Zetalytics...")
            res = zeta.ip(q=query)
            if "results" in res:
                for domain in res["results"]:
                    data["passive_dns"].append(
                        {
                            "domain": domain["qname"],
                            "source": "Zetalytics",
                            "first": parse(domain["date"]).astimezone(pytz.utc),
                            "last": parse(domain["last_seen"]).astimezone(pytz.utc)
                            if "last_seen" in domain
                            else None,
                        }
                    )



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="harpoon",
    version="0.1.7",
    description="Another OSINT CLI tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Te-k/harpoon",
    author="Tek",
    author_email="tek@randhome.io",
    keywords="osint threatintel",
    include_package_data=True,
    install_requires=[
        "requests",
        "configparser",
        "passivetotal>=2.5.9",
        "beautifulsoup4>=4.12.3",
        "lxml==6.1.0",
        "censys==2.2.0",
        "shodan",
        "fullcontact.py",
        "pyhunter",
        "PyGitHub>=1.55",
        "telethon==0.19.1.6",
        "virustotal-api",
        "pymisp==2.4.159",
        "OTXv2",
        "IPy",
        "maxminddb>=1.4.0",
        "pyasn",
        "spyonweb==0.1",
        "selenium",
        "geoip2",
        "dnspython",
        "consolemd==0.5.1",
        "pypermacc==0.1.1",
        "archiveis",
        "pytz",
        "pypdns==1.3",
        "pybinaryedge==0.5",
        "spyonweb==0.1",
        "pythreatgrid2==0.1.1",
        "pycrtsh==0.3.12",
        "pysafebrowsing==0.1.2",
        "pysecuritytrails==0.1.3",
        "phonenumbers==8.12.4",
        "threatminer==1.0",
        "zetalytics-api==1.0.1",
        "greynoise>=1.2.0",
        "pyhashlookup==1.2.1",
        "simplejson==3.17.6"
    ],
    python_requires=">=3.5",
    license="GPLv3",
    packages=["harpoon", "harpoon.commands", "harpoon.lib", "harpoon.data"],
    package_dir={"harpoon.lib": "harpoon/lib"},
    package_data={"harpoon": ["harpoon/data/*.conf"]},
    entry_points={"console_scripts": ["harpoon=harpoon.main:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#! /usr/bin/env python
import os
from io import BytesIO
from zipfile import ZipFile

import requests

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandUmbrella(Command):
    """
    # Umbrella

    **Check if a domain is in Umbrella Top 1 million domains**

    * `harpoon umbrella domain DOMAIN`
    * Check a list of domains in a file : `harpoon umbrella list FILE`
    """

    name = "umbrella"
    description = "Check if a domain is in Umbrella Top 1 million domains"
    topfile = os.path.join(os.path.expanduser("~"), ".config/harpoon/umbrellatop1m.csv")

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("domain", help="Check a domain on Umbrella")
        parser_a.add_argument("DOMAIN", help="DOMAIN to be queried")
        parser_a.set_defaults(subcommand="domain")
        parser_b = subparsers.add_parser("list", help="Check a list of domains")
        parser_b.add_argument("FILE", help="File containing list of domains")
        parser_b.set_defaults(subcommand="list")

        self.parser = parser

    def update(self):
        """
        Download file http://s3-us-west-1.amazonaws.com/umbrella-static/top-1m.csv.zip
        """
        print("Downloading Umbrella Top 1 million websites")
        if os.path.isfile(self.topfile):
            os.remove(self.topfile)
        r = requests.get(
            "http://s3-us-west-1.amazonaws.com/umbrella-static/top-1m.csv.zip"
        )
        if r.status_code != 200:
            print("Impossible to download Umbrella Top 1 millon CSV file.")
            return False
        input_zip = ZipFile(BytesIO(r.content))
        fname = input_zip.namelist()[0]
        with open(self.topfile, "a+") as f:
            f.write(input_zip.read(fname).decode("utf-8"))
        return True

    def check(self, domain):
        """
        Check if a domain is in the Umbrella Top 1 million list
        """
        if not os.path.isfile(self.topfile):
            print("Umbrella Top 1 million file not available, please do harpoon update")
            return None
        with open(self.topfile) as f:
            line = f.readline().strip()
            while line != "":
                ll = line.split(",")
                if domain == ll[1]:
                    return int(ll[0])
                line = f.readline().strip()
        return None

    def run(self, args, plugins):
        if "subcommand" in args:
            if args.subcommand == "domain":
                rank = self.check(unbracket(args.DOMAIN))
                if rank:
                    print("Found ranked {}".format(rank))
                else:
                    print("Not found")
            elif args.subcommand == "list":
                umbrella = {}
                with open(self.topfile) as f:
                    for line in f.read().split("\n"):
                        if line.strip() == "":
                            continue
                        ll = line.strip().split(",")
                        umbrella[ll[1]] = ll[0]

                with open(args.FILE) as f:
                    data = f.read().split("\n")
                data.remove("")

                for d in data:
                    if d.strip() == "":
                        continue
                    if d.strip() in umbrella.keys():
                        print(
                            "{} in the umbrella list at {} position".format(
                                d.strip(), umbrella[d.strip()]
                            )
                        )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_domain(self, query, data):
        rank = self.check(query)
        if rank:
            data["reports"].append(
                {
                    "date": "",
                    "title": "Domain ranked as {} by Cisco Umbrella".format(rank),
                    "url": "",
                    "source": "Cisco Umbrella",
                }
            )



#! /usr/bin/env python
import json
import sys

from pysafebrowsing.api import (SafeBrowsing, SafeBrowsingInvalidApiKey,
                                SafeBrowsingWeirdError)

from harpoon.commands.base import Command


class CommandSafeBrowsing(Command):
    """
    # Google Safe Browsing

    **Check online Safe Browsing information**

    * Query an url: `harpoon safebrowsing url URL`
    * Query a list of domains or url from a file with CSV output: `harpoon safebrowsing file FILE -f csv`
    """

    name = "safebrowsing"
    description = "Check if the given domain is in Google safe Browsing list"
    config = {"SafeBrowsing": ["key"]}

    def add_arguments(self, parser):
        # FIXME: migrate to subcommands
        subparsers = parser.add_subparsers(help="SubCommands")
        parser_b = subparsers.add_parser("url", help="Query an URL")
        parser_b.add_argument("URL", help="URL to be requested")
        parser_b.add_argument("--json", "-j", action="store_true", help="Show raw json")
        parser_b.set_defaults(subcommand="url")
        parser_c = subparsers.add_parser(
            "file", help="Check domains or urls from a file"
        )
        parser_c.add_argument("FILE", help="File path")
        parser_c.add_argument(
            "--format",
            "-f",
            help="Output format",
            choices=["json", "csv", "txt"],
            default="txt",
        )
        parser_c.set_defaults(subcommand="file")
        self.parser = parser

    def run(self, args, plugins):
        sb = SafeBrowsing(self._config_data["SafeBrowsing"]["key"])
        if "subcommand" in args:
            if args.subcommand == "url":
                try:
                    if args.URL.startswith("http"):
                        res = sb.lookup_url(args.URL)
                    else:
                        res = sb.lookup_url("http://" + args.URL + "/")
                except SafeBrowsingInvalidApiKey:
                    print("Invalid API key!")
                    sys.exit(1)
                except SafeBrowsingWeirdError:
                    print("Weird Error!")
                    sys.exit(1)
                else:
                    if args.json:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        if res["malicious"]:
                            print("Malicious: Yes")
                            print("Platforms: %s" % ", ".join(res["platforms"]))
                            print("Threats: %s" % ", ".join(res["threats"]))
                        else:
                            print("Malicious: No")
            elif args.subcommand == "file":
                with open(args.FILE, "r") as f:
                    data = f.read()
                domains = [d.strip() for d in data.split()]
                res = sb.lookup_urls(
                    [
                        "http://" + d + "/" if not d.startswith("http") else d
                        for d in domains
                    ]
                )
                if args.format == "txt":
                    for domain in res:
                        if res[domain]["malicious"]:
                            print("%s\tMalicious" % domain)
                        else:
                            print("%s\tOk" % domain)
                elif args.format == "json":
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    print("Url|Malicious|Threat|Platform")
                    for domain in res:
                        if res[domain]["malicious"]:
                            print(
                                "%s|%s|%s|%s"
                                % (
                                    domain,
                                    "Yes",
                                    ",".join(res[domain]["threats"]),
                                    ",".join(res[domain]["platforms"]),
                                )
                            )
                        else:
                            print("%s|No||" % domain)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests


class HibpError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class HibpNotFound(HibpError):
    def __init__(self):
        HibpError.__init__(self, "Account Not Found in the database")


class HIBP(object):
    """
    Object to request the Have I Been Pwned API
    https://haveibeenpwned.com/API/v2#BreachesForAccount
    """
    def __init__(self, key):
        self.ua = "Harpoon"
        self.key = key

    def _request(self, query):
        """
        Request an url
        """
        r = requests.get(
            "https://haveibeenpwned.com" + query,
            headers={'User-Agent': self.ua, 'hibp-api-key': self.key})
        if r.status_code != 200:
            if r.status_code == 404:
                raise HibpNotFound()
            else:
                raise HibpError('Invalid HTTP status code %i' % r.status_code)
        return r

    def get_breaches_account(self, account):
        return self._request('/api/v3/breachedaccount/%s' % account).json()

    def list_breaches(self):
        """List all breaches"""
        return self._request('/api/v3/breaches').json()

    def get_breach(self, breach):
        return self._request('/api/v3/breach/%s' % breach).json()

    def get_dataclasses(self):
        return self._request('/api/v3/dataclasses').json()

    def get_pastes(self, account):
        return self._request('/api/v3/pasteaccount/%s' % account).json()

    def check_pwd(self, pwd):
        return self._request('/api/v3/pwnedpassword/%s' % pwd).json()



#! /usr/bin/env python
import json

import pytz
from dateutil.parser import parse
from pythreatgrid2 import ThreatGrid, ThreatGridError

from harpoon.commands.base import Command
from harpoon.lib.utils import typeguess, unbracket


class CommandThreatGrid(Command):
    """
    # Threat Grid

    **Search in Threat Grid database through the API**

    * Search for a domain: `harpoon threatgrid search domain DOMAIN`
    * Search for a hash : ``harpoon threatgrid hash HASH`

    """

    name = "threatgrid"
    description = "Request Threat Grid API"
    config = {"ThreatGrid": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Request info on a hash")
        parser_a.add_argument("HASH", help="Hash")
        parser_a.add_argument("--json", "-j", action="store_true", help="Show raw json")
        parser_a.set_defaults(subcommand="hash")
        parser_b = subparsers.add_parser("search", help="Search in ThreatGrid database")
        parser_b.add_argument("--json", "-j", action="store_true", help="Show raw json")
        parser_b.add_argument(
            "TYPE",
            help="type of the query (can be checksum, checksum_sample, path,"
            + "path_sample, path_artifact, path_deleted, url, registry_key, "
            + "domain, domain_dns_lookup, domain_http_request, ip, ip_dns_lookup, ip_src, ip_dst, ioc, tag)",
        )
        parser_b.add_argument("QUERY", help="query")
        parser_b.set_defaults(subcommand="search")
        parser_c = subparsers.add_parser(
            "networklist",
            help="Read list of network indicators (domain or IP) from a file a search for related samples",
        )
        parser_c.add_argument("FILE", help="File")
        parser_c.set_defaults(subcommand="networklist")
        self.parser = parser

    def run(self, args, plugins):
        tg = ThreatGrid(self._config_data["ThreatGrid"]["key"])
        if "subcommand" in args:
            if args.subcommand == "search":
                try:
                    res = tg.search_samples(unbracket(args.QUERY), type=args.TYPE)
                except ThreatGridError:
                    print("Invalid type")
                if args.json:
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    if len(res["items"]) == 0:
                        print("Not found")
                    else:
                        already = []
                        for item in res["items"]:
                            if item["sample_sha256"] not in already:
                                print(
                                    "%s - %s - %s"
                                    % (
                                        item["ts"],
                                        item["sample_sha256"],
                                        "https://panacea.threatgrid.com/mask/samples/"
                                        + item["sample"],
                                    )
                                )
                                already.append(item["sample_sha256"])
            elif args.subcommand == "hash":
                hash_type = {32: "md5", 40: "sha1", 64: "sha256"}
                res = tg.get_sample(args.HASH, type=hash_type[len(args.HASH)])
                if len(res["items"]) > 0:
                    item = res["items"][0]
                    print(
                        "Sample submitted the %s: https://panacea.threatgrid.com/mask/samples/%s"
                        % (item["submitted_at"], item["id"])
                    )
                    idd = item["id"]
                    res = tg.get_sample_threats(idd)
                    if args.json:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        print("\nThreats:")
                        for t in res["bis"]:
                            print("-%s" % t)
                else:
                    print("Hash not found")
            elif args.subcommand == "networklist":
                with open(args.FILE, "r") as f:
                    data = f.read().split("\n")
                for d in data:
                    target = unbracket(d.strip())
                    gtype = typeguess(target)
                    print(target)
                    res = tg.search_samples(target, type=gtype)
                    if len(res["items"]) > 0:
                        already = []
                        for item in res["items"]:
                            if item["sample_sha256"] not in already:
                                print(
                                    "-%s: https://panacea.threatgrid.com/mask/samples/%s %s"
                                    % (
                                        item["ts"][:10],
                                        item["sample"],
                                        item["sample_sha256"],
                                    )
                                )
                                already.append(item["sample_sha256"])

                    else:
                        print("-Nothing found")
                    print("")
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "domain":
            print("[+] Checking ThreatGrid...")
            try:
                tg = ThreatGrid(self._config_data["ThreatGrid"]["key"])
                res = tg.search_samples(query, type="domain")
                already = []
                if "items" in res:
                    for r in res["items"]:
                        if r["sample_sha256"] not in already:
                            d = parse(r["ts"]).astimezone(pytz.utc)
                            data["malware"].append(
                                {
                                    "hash": r["sample_sha256"],
                                    "date": d,
                                    "source": "ThreatGrid",
                                }
                            )
                            already.append(r["sample_sha256"])
            except ThreatGridError as e:
                print("Failed to connect to Threat Grid: %s" % e.message)
        elif type == "ip":
            print("[+] Checking ThreatGrid...")
            try:
                tg = ThreatGrid(self._config_data["ThreatGrid"]["key"])
                res = tg.search_samples(query, type="ip")
                already = []
                if "items" in res:
                    for r in res["items"]:
                        if r["sample_sha256"] not in already:
                            d = parse(r["ts"]).astimezone(pytz.utc)
                            data["malware"].append(
                                {
                                    "hash": r["sample_sha256"],
                                    "date": d,
                                    "source": "ThreatGrid",
                                }
                            )
                            already.append(r["sample_sha256"])
            except ThreatGridError as e:
                print("Failed to connect to Threat Grid: %s" % e.message)
        elif type == "hash":
            print("[+] Checking ThreatGrid...")
            try:
                tg = ThreatGrid(self._config_data["ThreatGrid"]["key"])
                hash_type = {32: "md5", 40: "sha1", 64: "sha256"}
                res = tg.get_sample(query, type=hash_type[len(query)])
                for item in res["items"]:
                    data["samples"].append(
                        {
                            "source": "ThreatGrid",
                            "date": parse(item["submitted_at"]).astimezone(pytz.utc),
                            "url": "https://panacea.threatgrid.com/mask/samples/{}".format(
                                item["id"]
                            ),
                        }
                    )
            except ThreatGridError as e:
                print("Failed to connect to Threat Grid: %s" % e.message)



#! /usr/bin/env python
import json
from datetime import datetime

from harpoon.commands.base import Command
from harpoon.lib.koodous import Koodous, KoodousError, KoodousNotFound


class CommandKoodous(Command):
    """
    # Koodous plugin

    Queries the Koodous API https://koodous.com/

    * get info on a hash : `harpoon koodous hash SHA256`
    * Download a file : `harpoon koodous dl SHA256`
    """

    name = "koodous"
    description = "Request Koodous API"
    config = {"Koodous": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Get info on a SHA256 hash")
        parser_a.add_argument("HASH", help="SHA256 hash")
        parser_a.set_defaults(subcommand="hash")
        parser_b = subparsers.add_parser("search", help="Search in Koodous")
        parser_b.add_argument("QUERY", help="Query")
        parser_b.set_defaults(subcommand="search")
        parser_c = subparsers.add_parser("dl", help="Download a sample from Koodous")
        parser_c.add_argument("HASH", help="Sha256")
        parser_c.set_defaults(subcommand="dl")
        parser_d = subparsers.add_parser(
            "analysis", help="Get a full analysis from Koodous"
        )
        parser_d.add_argument("HASH", help="Sha256")
        parser_d.set_defaults(subcommand="analysis")
        parser_e = subparsers.add_parser(
            "account", help="Get information on the account"
        )
        parser_e.set_defaults(subcommand="account")
        self.parser = parser

    def run(self, args, plugins):
        kd = Koodous(token=self._config_data["Koodous"]["token"])
        if "subcommand" in args:
            try:
                if args.subcommand == "hash":
                    res = kd.sha256(args.HASH)
                    print(json.dumps(res, indent=4))
                elif args.subcommand == "search":
                    res = kd.search(args.QUERY)
                    print(json.dumps(res, indent=4))
                elif args.subcommand == "dl":
                    data = kd.download(args.HASH)
                    with open(args.HASH, "wb+") as f:
                        f.write(data)
                    print("File downloaded as {}".format(args.HASH))
                elif args.subcommand == "analysis":
                    res = kd.analysis(args.HASH)
                    print(json.dumps(res, indent=4))
                elif args.subcommand == "account":
                    res = kd.account()
                    print(json.dumps(res, indent=4))
                else:
                    self.parser.print_help()
            except KoodousNotFound:
                print("Not found")
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "hash":
            if len(query) == 64:
                try:
                    kd = Koodous(token=self._config_data["Koodous"]["token"])
                    res = kd.sha256(query)
                except KoodousError:
                    pass
                else:
                    data["samples"].append(
                        {
                            "source": "Koodous",
                            "date": datetime.fromtimestamp(res["created_on"]),
                            "url": "https://koodous.com/apks/" + query,
                        }
                    )



import archiveis
import requests
from dateutil.parser import parse

from harpoon.lib.memento import MementoClient


class ArchiveIs(object):
    """
    Class to request achive.is website
    """
    @staticmethod
    def snapshots(url):
        """
        Return all the screenshot of an url
        """
        mc = MementoClient(base_url='http://archive.is/')
        return mc.snapshots(url)

    @staticmethod
    def download_cache(cache_url):
        """
        return cache data from an archive.is cached url
        """
        r = requests.get(cache_url)
        data = r.text
        t1 = data.find('\n\n\n\n\n\n')
        t2 = data.find(
            '</div></div><!--[if !IE]><!--><div style="position:absolute;right:1028px;' +
            'top:-14px;bottom:-2px">')
        # t3 = data.find('<input style="border:1px solid black;height:20px;' +
        #  'margin:0 0 0 0;padding:0;width:500px" type="text" name="q"')
        # t4 = data[t3+115:].find('"')
        t5 = data.find('<meta property="article:modified_time" content="')
        cached_data = data[t1+6:t2]
        # original_url = data[t3+115:t3+115+t4]
        date = parse(data[t5+48:t5+68])
        return {
            'success': True,
            'date': date,
            'data': cached_data,
            'cacheurl': cache_url
        }

    @staticmethod
    def cache(url):
        """
        Get a cache url and download the last one
        """
        snapshots = ArchiveIs.snapshots(url)
        if len(snapshots) > 0:
            last = sorted(snapshots, key=lambda x: x['date'], reverse=True)[0]
            return ArchiveIs.download_cache(last['archive'])
        else:
            return {
                'success': False
            }

    @staticmethod
    def capture(url):
        """
        Capture an url in archive.is
        """
        # Easiest way to do it for now, archive.is API sucks
        # FIXME replace this lib
        return archiveis.capture(url)



#! /usr/bin/env python
from collections import Counter
from urllib.parse import urljoin

import pytz
from dateutil.parser import parse
from pymisp import ExpandedPyMISP

from harpoon.commands.base import Command


class CommandMisp(Command):
    """
    # MISP

    **Queries a MISP (http://www.misp-project.org/) server**

    * List events (maybe a long list): `harpoon misp -l`
    * Get information on an event: `harpoon misp -e 342`
    * Get all the domain IOCs from an event: `harpoon misp -e 42 -t domain`
    * Search for an attribute: `harpoon misp -a example.org`
    """

    name = "misp"
    description = "Get information from a MISP server through the API"
    config = {"Misp": ["url", "key"]}

    def add_arguments(self, parser):
        parser.add_argument("--list", "-l", action="store_true", help="List events")
        parser.add_argument("--event", "-e", help="Event infos", type=int)
        parser.add_argument("--attr", "-a", help="Search for this attribute")
        parser.add_argument("--type", "-t", help="Search for attributes of this type")
        parser.add_argument(
            "--raw", "-r", help="Print raw information", action="store_true"
        )
        parser.add_argument(
            "--no-tls",
            "-n",
            help="Do not validate TLS certificate (bad bad bad)",
            action="store_true",
        )
        self.parser = parser

    def run(self, args, plugins):
        server = ExpandedPyMISP(
            self._config_data["Misp"]["url"],
            self._config_data["Misp"]["key"],
            not args.no_tls,
        )
        if args.list:
            # List events
            events = server.events(pythonify=True)
            for event in sorted(events, key=lambda x: x.id):
                print("%i : %s" % (event.id, event.info))
        elif args.event is not None:
            event = server.get_event(args.event, pythonify=True)
            if args.attr is None and args.type is None:
                if args.raw:
                    for a in event.attributes:
                        print(a.value)
                else:
                    print("Event {} : {}".format(event.id, event.info))
                    print(
                        "Tags : {}".format(
                            ", ".join(map(lambda x: str(x.name), event.tags))
                        )
                    )
                    print("{} Attributes including:".format(len(event.attributes)))
                    attrs = Counter(map(lambda x: x.type, event.attributes))
                    attrs_ids = Counter(
                        map(
                            lambda x: x.type,
                            filter(lambda x: x.to_ids, event.attributes),
                        )
                    )
                    for type in attrs:
                        print(
                            "- %i %s (%i for detection)"
                            % (attrs[type], type, attrs_ids[type])
                        )
            else:
                if args.type is not None:
                    # Display all attributes from this type
                    for attr in event.attributes:
                        if attr.type == args.type:
                            if args.raw:
                                print("%s" % attr.value)
                            else:
                                print(
                                    "{:20}{:10}{:40}{}{}".format(
                                        attr.category,
                                        attr.type,
                                        attr.value,
                                        attr.comment,
                                        attr.to_ids,
                                    )
                                )
                elif args.attr is not None:
                    # search by attribute value
                    for attr in event.attributes:
                        if args.attr in str(attr.value):
                            print(
                                "%s\t%s\t%s\t%s\t%s"
                                % (
                                    attr.category,
                                    attr.type,
                                    attr.value,
                                    attr.comment,
                                    attr.to_ids,
                                )
                            )
        elif args.attr is not None:
            res = server.search("attributes", value=args.attr)
            if len(res["Attribute"]) == 0:
                print("Search %s: no results" % args.attr)
            else:
                print("Search %s, result founds" % args.attr)
                for attr in res["Attribute"]:
                    print("{} - {}".format(attr["Event"]["id"], attr["Event"]["info"]))
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type in ["domain", "ip", "hash"]:
            print("[+] Checking MISP...")
            server = ExpandedPyMISP(
                self._config_data["Misp"]["url"], self._config_data["Misp"]["key"]
            )
            misp_results = server.search("events", value=query)
            for event in misp_results:
                data["reports"].append(
                    {
                        "date": parse(event["Event"]["date"]).astimezone(pytz.utc),
                        "title": event["Event"]["info"],
                        "source": "MISP",
                        "url": urljoin(
                            self._config_data["Misp"]["url"],
                            "events/view/{}".format(event["Event"]["id"]),
                        ),
                    }
                )



import requests


class CyberCureError(Exception):
    def __init__(self, message):
        self.message = message
        Exception.__init__(self, message)


class CyberCure(object):
    def __init__(self, token):
        self.token = token
        self.base_url = 'http://api.cybercure.ai/feed/search?value='

    def get_infos(self, ip):
        r = requests.get(self.base_url + ip, headers={'User-Agent': 'harpoon (https://github.com/Te-k/harpoon/)'})
        if r.status_code != 200:
            raise CyberCureError('Invalid HTTP code %i' % r.status_code)
        return r.json()



#! /usr/bin/env python
import json
import time

import censys
from censys.search import CensysCerts, CensysHosts

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandCensys(Command):
    """
    # Censys plugin

    **Query Censys.io API**

    * Query information on an IP: `harpoon censys ip 172.217.2.174`
    * Query a certificate: `harpoon censys certificate ID`
    * Search for hosts: `harpoon censys search QUERY -p 20`
    """

    name = "censys"
    description = "Request information from Censys database (https://censys.io/)"
    config = {"Censys": ["id", "secret"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("ip", help="Get information on an IP address")
        parser_a.add_argument("IP", help="IP to be searched")
        parser_a.add_argument(
            "--events", "-e", action="store_true", help="Show events for this IP"
        )
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser(
            "cert", help="Get information on a certificate"
        )
        parser_b.add_argument("ID", help="ID of the certificate")
        parser_b.set_defaults(subcommand="cert")
        parser_e = subparsers.add_parser(
            "search", help="Search for hosts using Censys V2 syntax"
        )
        parser_e.add_argument("QUERY", help="Censys v2 query")
        parser_e.add_argument(
            "--pages",
            "-p",
            default=10,
            type=int,
            help="Number of pages (100 results per page, each page costs 1 quota)",
        )
        parser_e.add_argument(
            "--verbose",
            "-v",
            action="store_true",
            help="Verbose mode (display more than just the IP address)",
        )
        parser_e.add_argument(
            "--file", "-f", action="store_true", help="Read the query from a file"
        )
        parser_e.add_argument(
            "--output", "-o", help="Output file (stdout if not provided)"
        )
        parser_e.set_defaults(subcommand="search")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            if args.subcommand == "ip":
                api = CensysHosts(
                    self._config_data["Censys"]["id"],
                    self._config_data["Censys"]["secret"],
                )
                if args.events:
                    res = api.view_host_events(unbracket(args.IP))
                    print(json.dumps(res, indent=4, separators=(",", ": ")))
                else:
                    try:
                        ip = api.view(unbracket(args.IP))
                        print(json.dumps(ip, indent=4, separators=(",", ": ")))
                    except censys.base.CensysNotFoundException:
                        print("IP not found")
            elif args.subcommand == "cert":
                try:
                    print(
                        "Viewing certs is not implemented yet, seeing hosts for this cert:"
                    )
                    c = CensysCerts(
                        self._config_data["Censys"]["id"],
                        self._config_data["Censys"]["secret"],
                    )
                    res = c.get_hosts_by_cert(args.ID)
                except censys.base.CensysNotFoundException:
                    print("Certificate not found")
                else:
                    print(
                        json.dumps(
                            res, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            elif args.subcommand == "search":
                api = CensysHosts(
                    self._config_data["Censys"]["id"],
                    self._config_data["Censys"]["secret"],
                )
                if args.file:
                    with open(args.QUERY) as f:
                        query = f.read().strip()
                else:
                    query = args.QUERY
                print("Searching for {}".format(query))
                if args.output:
                    fout = open(args.output, "w+")
                    total = 0
                for page in api.search(query, per_page=100, pages=args.pages):
                    if args.output:
                        for host in page:
                            if args.verbose:
                                fout.write(
                                    "{},{},{},{}\n".format(
                                        host["ip"],
                                        host["location"]["country"],
                                        host["autonomous_system"]["asn"],
                                        host["autonomous_system"]["name"],
                                    )
                                )
                            else:
                                fout.write("{}\n".format(host["ip"]))
                        total += len(page)
                        print("{} ips written in {}".format(total, args.output))
                    else:
                        for host in page:
                            if args.verbose:
                                try:
                                    print(
                                        "{} - [{}] - [{}]".format(
                                            host["ip"],
                                            ", ".join(
                                                [
                                                    str(a["port"])
                                                    + "/"
                                                    + a["service_name"]
                                                    for a in host["services"]
                                                ]
                                            ),
                                            host["autonomous_system"]["asn"]
                                            + " / "
                                            + host["autonomous_system"]["name"],
                                        )
                                    )
                                except KeyError:
                                    print(host["ip"])
                            else:
                                print(host["ip"])
                    # To avoid rate limiting
                    time.sleep(0.5)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "ip":
            print("[+] Checking Censys...")
            api = CensysHosts(
                self._config_data["Censys"]["id"], self._config_data["Censys"]["secret"]
            )
            ip = api.view(query)
            for service in ip["services"]:
                data["ports"].append(
                    {
                        "port": service["port"],
                        "info": service["service_name"],
                        "source": "Censys",
                    }
                )

    def get_subdomains(self, conf, query, verbose):
        api = CensysHosts(
            self._config_data["Censys"]["id"], self._config_data["Censys"]["secret"]
        )
        raw = api.search(query)
        cleaned = raw.view_all()
        for host in cleaned:
            for i in cleaned[host]["services"]:
                try:
                    leaf_data = i["tls"]["certificates"]["leaf_data"]["names"]
                except KeyError:
                    pass

        return leaf_data



import requests


class ThreatCrowdError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class ThreatCrowd(object):
    """
    ThreatCrowd API handler
    https://github.com/AlienVault-OTX/ApiV2
    """
    def __init__(self):
        self.base_url = "http://www.threatcrowd.org/searchApi/v2/"

    def _query(self, queryType, query):
        if queryType == "file":
            res = requests.get(
                self.base_url + queryType + "/report/", {"resource": query}
            )
        else:
            res = requests.get(
                self.base_url + queryType + "/report/", {queryType: query}
            )
        if res.status_code != 200:
            raise ThreatCrowdError("Erorr: HTTP code {}".format(res.status_code))
        return res.json()

    def ip(self, ip):
        """
        get info on an IP address
        """
        return self._query("ip", ip)

    def email(self, email):
        return self._query("email", email)

    def domain(self, domain):
        return self._query("domain", domain)

    def antivirus(self, av):
        return self._query("antivirus", av)

    def file(self, ffile):
        return self._query("file", ffile)



#! /usr/bin/env python
import requests

from harpoon.commands.base import Command
from harpoon.lib.utils import is_ip, unbracket


class CommandTor(Command):
    """
    # Tor

    **Check if an IP is a Tor exit node listed in the public list https://check.torproject.org/torbulkexitlist**

    * `harpoon tor IP`
    """

    name = "tor"
    description = "Check if an IP is a Tor exit node listed in the public list"
    config = {"Tor": []}

    def add_arguments(self, parser):
        parser.add_argument("IP", help="IP Address")
        self.parser = parser

    def get_list(self):
        r = requests.get("https://check.torproject.org/torbulkexitlist")
        if r.status_code == 200:
            res = r.text.split("\n")
            try:
                res.remove("")
            except ValueError:
                pass
            return res
        return None

    def run(self, args, plugins):
        if not is_ip(unbracket(args.IP)):
            print("Invalid IP address")
            return
        ips = self.get_list()
        if ips:
            if unbracket(args.IP) in ips:
                print("{} is a Tor Exit node".format(unbracket(args.IP)))
            else:
                print(
                    "{} is not listed in the Tor Exit node public list".format(
                        unbracket(args.IP)
                    )
                )
        else:
            print("Impossible to reach the Tor Exit node list")

    def intel_ip(self, query, data):
        print("[+] Checking Tor exit nodes...")
        ips = self.get_list()
        if query.strip() in ips:
            data["reports"].append(
                {
                    "date": None,
                    "title": "{} is a Tor Exit Node".format(query),
                    "url": "",
                    "source": "TorExit",
                }
            )



#! /usr/bin/env python
import json
import sys

from pyhunter import PyHunter

from harpoon.commands.base import Command


class CommandHunter(Command):
    """
    # Hunter.io

    **Requests Hunter.io plugin https://hunter.io/**

    * Search for an email: `harpoon hunter email FIRSTNAME LASTNAME DOMAIN`
    * Search for a domain: `harpoon hunder domain example.org`
    """

    name = "hunter"
    description = "Request hunter.io information through the API"
    config = {"Hunter": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("email")
        parser_a.add_argument("NAME", help="Name of the user")
        parser_a.add_argument("DOMAIN", help="Domain of the user")
        parser_a.set_defaults(subcommand="email")
        parser_a = subparsers.add_parser("domain")
        parser_a.add_argument("DOMAIN", help="Domain to look for")
        parser_a.set_defaults(subcommand="domain")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            hunter = PyHunter(self._config_data["Hunter"]["key"])
            if args.subcommand == "email":
                if " " not in args.NAME:
                    print("Name should contains first and last name")
                    print("(Yes this API is useless)")
                    sys.exit(1)
                res = hunter.email_finder(
                    domain=args.DOMAIN, full_name=args.NAME, raw=True
                )
                print(json.dumps(res, sort_keys=True, indent=4))
            elif args.subcommand == "domain":
                res = hunter.domain_search(args.DOMAIN)
                print(json.dumps(res, sort_keys=True, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import base64

import requests


class XforceExchangeFailed(Exception):
    pass


class XforceExchangeNotFound(XforceExchangeFailed):
    pass


class XforceExchange(object):
    def __init__(self, api_key, password):
        self.api_key = api_key
        self.password = password
        self.base_url = "https://api.xforce.ibmcloud.com:443"
        self.ua = "Harpoon https://github.com/Te-k/harpoon"

    def _request(self, url, params={}):
        token = base64.b64encode(self.api_key.encode('utf-8') + b":" + self.password.encode('utf-8'))
        headers = {
            'Authorization': "Basic " + token.decode('utf-8'),
            'Accept': 'application/json',
            'User-Agent': self.ua
        }
        r = requests.get(
            self.base_url + url,
            params=params,
            headers=headers
        )
        if r.status_code == 200:
            return r.json()
        else:
            if r.status_code == 404:
                raise XforceExchangeNotFound()
            else:
                print(r.text)
                raise XforceExchangeFailed()

    def ip_reputation(self, ip):
        """
        Returns the IP reputation report for the entered IP.
        """
        return self._request('/ipr/history/' + ip)

    def ip(self, ip):
        """
        Returns the IP report for the entered IP.
        """
        return self._request('/ipr/' + ip)

    def ip_malware(self, ip):
        """
        Returns the malware associated with the entered IP.
        """
        return self._request('/ipr/malware/' + ip)

    def search(self, query):
        """
        Returns a list of public Collections that were found
        """
        return self._request('/casefiles/public/fulltext', params={'q': query})

    def dns(self, _input):
        """
        Returns live and passive DNS records.
        """
        return self._request('/resolve/' + _input)

    def casefile(self, _id):
        """
        Returns a JSON resentation of a Collection
        """
        return self._request('/casefiles/' + _id)

    def malware(self, _hash):
        """
        Returns a malware report for the given file hash, For example, md5,
        sha1 and sha256.
        """
        return self._request('/malware/' + _hash)

    def url(self, url):
        """
        Returns the URL report for the entered URL.
        """
        return self._request('/url/' + url)

    def usage(self):
        """
        Get API usage details per month for each subscription type.
        """
        return self._request('/all-subscriptions/usage')

    def whois(self, domain):
        """
        Returns a JSON object containing information about the given host address.
        """
        return self._request('/whois/' + domain)



#! /usr/bin/env python
import json

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket
from harpoon.lib.xforce import XforceExchange, XforceExchangeNotFound


class CommandXforce(Command):
    """
    # IBM Xforce Exchange

    Query IBM Xforce Exchange API
    """

    name = "xforce"
    description = "Query IBM Xforce Exchange API"
    config = {"Xforce": ["key", "password"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("ip_reputation", help="Get IP Reputation")
        parser_a.add_argument("IP", help="IP address")
        parser_a.set_defaults(subcommand="ip_reputation")
        parser_b = subparsers.add_parser("ip", help="Get information on an IP")
        parser_b.add_argument("IP", help="IP address")
        parser_b.set_defaults(subcommand="ip")
        parser_c = subparsers.add_parser(
            "ip_malware", help="Returns the malware associated with the entered IP."
        )
        parser_c.add_argument("IP", help="IP address")
        parser_c.set_defaults(subcommand="ip_malware")
        parser_d = subparsers.add_parser(
            "search", help="Returns a list of public Collections that were found"
        )
        parser_d.add_argument("QUERY", help="Query")
        parser_d.set_defaults(subcommand="search")
        parser_e = subparsers.add_parser(
            "dns", help="Returns live and passive DNS records."
        )
        parser_e.add_argument("INPUT", help="Input")
        parser_e.set_defaults(subcommand="dns")
        parser_f = subparsers.add_parser(
            "casefile", help="Returns a JSON resentation of a Collection"
        )
        parser_f.add_argument("ID", help="ID of a casefile")
        parser_f.set_defaults(subcommand="casefile")
        parser_g = subparsers.add_parser(
            "malware",
            help="Returns a malware report for the given file hash, For example, md5, sha1 and sha256.",
        )
        parser_g.add_argument("HASH", help="Hash of a malware")
        parser_g.set_defaults(subcommand="malware")
        parser_h = subparsers.add_parser(
            "url", help="Returns the URL report for the entered URL."
        )
        parser_h.add_argument("URL", help="Url")
        parser_h.set_defaults(subcommand="url")
        parser_i = subparsers.add_parser("usage", help="Get API usage details")
        parser_i.set_defaults(subcommand="usage")
        parser_j = subparsers.add_parser(
            "whois", help="Returns whois information on a domain"
        )
        parser_j.add_argument("DOMAIN", help="Domain")
        parser_j.set_defaults(subcommand="whois")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            xe = XforceExchange(
                self._config_data["Xforce"]["key"],
                self._config_data["Xforce"]["password"],
            )
            if args.subcommand == "ip_reputation":
                res = xe.ip_reputation(unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ip":
                res = xe.ip(unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "ip_malware":
                res = xe.ip_malware(unbracket(args.IP))
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "search":
                res = xe.search(args.QUERY)
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "dns":
                res = xe.dns(args.INPUT)
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "casefile":
                res = xe.casefile(args.ID)
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "malware":
                try:
                    res = xe.malware(args.HASH)
                    print(json.dumps(res, sort_keys=False, indent=4))
                except XforceExchangeNotFound:
                    print("Not found")
            elif args.subcommand == "url":
                res = xe.url(args.URL)
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "usage":
                res = xe.usage()
                print(json.dumps(res, sort_keys=False, indent=4))
            elif args.subcommand == "whois":
                res = xe.whois(unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=False, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    # TODO : intel



import argparse
import configparser
import os
import signal
import sys

from harpoon.commands.base import Command


def handle_stop(sig, frame):
    print('Okay, Okay, I stop...')
    sys.exit(1)


def load_config():
    config = configparser.ConfigParser()
    if os.path.isfile(os.path.join(os.path.expanduser("~"), ".config/harpoon/config")):
        config.read(os.path.join(os.path.expanduser("~"), ".config/harpoon/config"))
    return config


def init_plugins(config):
    """
    Find all the plugins available and init them
    """
    plugin_dir = os.path.dirname(os.path.realpath(__file__)) + '/commands'
    plugin_files = [x[:-3] for x in os.listdir(plugin_dir) if x.endswith(".py")]
    sys.path.insert(0, plugin_dir)
    for plugin in plugin_files:
        mod = __import__(plugin)  # noqa: F841

    PLUGINS = {}
    for plugin in Command.__subclasses__():
        PLUGINS[plugin.name] = plugin(config)
    return PLUGINS


def main():
    signal.signal(signal.SIGINT, handle_stop)
    config = load_config()

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help='Commands')

    # Init plugins
    plugins = init_plugins(config)
    for p in sorted(plugins.keys()):
        sp = subparsers.add_parser(
            plugins[p].name,
            help=plugins[p].description
        )
        plugins[p].add_arguments(sp)
        sp.set_defaults(command=p)

    args = parser.parse_args()

    # Run the plugin
    if hasattr(args, 'command'):
        # Config plugin need plugin list
        if not plugins[args.command].test_config():
            print('Invalid configuration for this plugin, quitting...')
            sys.exit(1)
        else:
            # FIXME : ugly
            if args.command == "help":
                plugins[args.command].run(args, plugins, parser)
            else:
                plugins[args.command].run(args, plugins)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()
